# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/util-time"

module Qwik
  class Action
    #------------------------------------------------------------ basic
    def plg_br
      [:br]
    end

    def plg_null
      ""
    end

    def plg_test
      "test"
    end

    def plg_menu(cmd, msg=nil)
      return "" if @req.user.nil?
      return "" if !defined?(@req.base) || @req.base.nil?
      case cmd
      when "newpage"
	return [:a, {:href=>".new"}, _("newpage")]
      when "edit"
	return [:a, {:href=>@req.base+".edit"}, _("edit")]
      when "wysiwyg"
	return [:a, {:href=>@req.base+".wysiwyg"}, _("wysiwyg")]
      else
	return ""
      end
    end

    def plg_last_modified
      return "" if !defined?(@req.base) || @req.base.nil?
      page = @site[@req.base]
      return "" if page.nil?
      date = page.mtime
      [:span, {:class=>"attribute"}, _("Last modified"), ": ",
	[:em, date.ymd]]
    end

    def plg_generate_time
      return "" if @req.user.nil?
      diff = Time.now - @req.start_time
      diffsec = sprintf("%.2f", diff)
      [:span, {:class=>"attribute"}, _("Generate time"), ": ",
	[:em, diffsec, _("sec.")]]
    end

    def plg_show_sitemenu
      [:span, {:class=>"attribute"},
	[:a, {:href=>"_SiteMenu.html"}, _("SiteMenu")]]
    end

    def plg_only_guest
      return if @req.user
      s = yield
      return if s.nil?
      c_res(s)
    end

    def plg_only_member
      return if @req.user.nil?
      s = yield
      return if s.nil?
      c_res(s)
    end

    def plg_window(url, text=nil)
      text = url if text.nil?
      [:a, {:href=>url, :target=>"_blank"}, text]
    end

    #------------------------------------------------------------ list
    def act_list
      str = @site.title_list.map {|page|
	"- "+page.mtime.ymd+" : [[#{page.key}]]"
      }.join("\n")
      msg = _("Page List")
      c_view(msg){str}
    end

    def act_recent(max = -1)
      msg = _("Recent Changes")
      str = recent_str(max)
      c_view(msg){str}
    end

    def plg_recent(max = -1)
      c_res(recent_str(max))
    end

    def plg_srecent(max = -1)
      max = max.to_i
      ar = []
      now = Time.now
      list = @site.date_list
      list.reverse.each_with_index {|page, i|
	break if 0 <= max && max < (i += 1)
	time = page.mtime
	difftime = now.to_i - time.to_i
	timestr = time.ymdx.to_s
	li = [:li,
	  [:a, {:href=>page.key+".html", :title=>timestr}, page.get_title]]
	if @req.user
	  li += [" ", [:span, {:class=>"ago"}, int_to_time(difftime)+_(" ago")]]
	end
	ar << li
      }
      if max < list.length
	ar << [:li, [:a, {:href=>".recent"}, [:em, _("more...")]]]
      end
      [:div, {:class=>"recent"}, [:ul, ar]]
    end

    def recent_str(max = -1)
      max = max.to_i
      ar = []
      last_day = nil
      add_day = true
      @site.date_list.reverse.each_with_index {|page, i|
	break if 0 <= max && max < (i += 1)
	if add_day
	  day = page.mtime.ymd.to_s
	  if last_day.nil? || day != last_day
	    ar << "** #{day}"
	    last_day = day
	  end
	end
	ar << "- [[#{page.key}]]"
      }
      ar.join("\n")
    end

    def int_to_time(n)
      return n.to_s+_("sec.") if n < 60 # under 1 min 
      return (n/60).to_s+_("min.") if n < 60*60 # under 1 hour
      return (n/(60*60)).to_s+_("hour") if n < 60*60*24 # under 1 day
      return (n/(60*60*24)).to_s+_("day") if n < 60*60*24*30 # under 1 month
      return (n/(60*60*24*30)).to_s+_("month") if n < 60*60*24*365 # under 1 year
      return (n/(60*60*24*365)).to_s+_("year") if n < 60*60*24*365*100 # under 1 century
      return (n/(60*60*24*365*100)).to_s+_("century")
    end

    #------------------------------------------------------------
    def plg_mail(from="", time=nil)
      str = yield
      h = ""
      h = "{{date("+time.to_s+")}}" if time
      h += " From: "+MailAddress.new(from).obfuscate
      str = MailAddress.obfuscate(str)
      msg = "* "+h+"\n"+str
      c_pre_text(msg)
    end

    def plg_include(pagename)
      page = @site[pagename.to_s]
      return "" if page.nil?
      str = page.load

      org_base = @req.base
      @req.base = pagename

      w = c_res(str)
      tres = TDiaryResolver.new(@config, @site, self)
      w = tres.resolve(w)

      @req.base = org_base
      w
    end

    def plg_text(str="")
      str = "" if str.nil?
      if block_given?
	str2 = yield
	str += str2 if str2
      end
      c_res(str)
    end

    def plg_date(time=nil)
      ts = ""
      if time.is_a? String
	if /\A[0-9]+\z/ =~ time
	  t = Time.at(time.to_i)
	  ts = t.strftime("%b #{t.day}, %Y")
	elsif /\A[-0-9]+\z/ =~ time
	  ts = time
	else
	  return ""
	end
      end
      [:span, {:class=>"date"}, ts]
    end

  end
end

if $0 == __FILE__
  require "t/test"
  $debug = true
end

if $debug
  class TC_ActBasicPlugin < Test::Unit::TestCase
    include TestSession

    def assert_time(e, n)
      assert_equal(e, @action.int_to_time(n))
    end

    def test_basic
      assert_wiki("", "{{null}}")
      assert_wiki("test", "{{test}}")

      # test_adminmenu
      # not work for now
      #assert_wiki("", "{{menu(newpage)}}", nil)
      #assert_wiki("", "{{menu(edit)}}", nil)
      #assert_wiki("", "{{menu(nosuchmenu)}}", nil)
      assert_wiki("<a href=\".new\">newpage</a>", "{{menu(newpage)}}")
      assert_wiki("<a href=\".new\">newpage</a>", "{{menu(newpage)}}")
      assert_wiki("<a href=\"1.edit\">edit</a>", "{{menu(edit)}}")
      assert_wiki("", "{{menu(nosuchmenu)}}")

      # test_last_modified
      assert_wiki(/Last modified: /, "{{last_modified}}")

      # test_generate_time
      assert_wiki(/Generate time: /, "{{generate_time}}")

      # test_show_sitemenu
      assert_wiki(/SiteMenu/, "{{show_sitemenu}}")

      # test_only_member_or_guest
      assert_wiki("<p>m</p>", "{{only_member\nm\n}}")
      assert_wiki("", "{{only_member\nm\n}}", nil)
      assert_wiki("", "{{only_guest\ng\n}}")
      assert_wiki("<p>g</p>", "{{only_guest\ng\n}}", nil)

      # test_window
      assert_wiki("<a href=\"url\" target=\"_blank\">t</a>",
		  "{{window(url,t)}}")
      assert_wiki("<a href=\"url\" target=\"_blank\">url</a>",
		  "{{window(url)}}")

      # test date
      assert_wiki("<span class=\"date\">2001-01-01</span>",
		  "{{date(2001-01-01)}}")
      assert_wiki("<span class=\"date\">Jan 1, 1970</span>",
		  "{{date(0)}}")
      assert_wiki("<span class=\"date\">2001-01-01</span>",
		  "{{date(2001-01-01)}}")
      assert_wiki("", "{{date(a)}}")

      # test_list
      t_add_user

      session("/test/.list")
      assert_text("Page List", "title")

      session("/test/.recent")
      assert_text("Recent Changes", "title")

      #assert_wiki(/<ul><li>/, "{{list}}")
      #assert_wiki(%r| : <a href="1.html">t</a></li>|, "{{list}}")
      assert_wiki(/<h3>/, "{{recent}}")
      assert_wiki(%r|<li><a href=\"1.html\">1</a></li>|, "{{recent}}")
      assert_wiki(/<h3>/, "{{recent(1)}}")

      # test_time
      assert_time("1sec.", 1)
      assert_time("1min.", 60)
      assert_time("1hour", 60*60)
      assert_time("1day", 60*60*24)
      assert_time("1month", 60*60*24*30)
      assert_time("1year", 60*60*24*365)
      assert_time("1century", 60*60*24*365*100)
    end
  end

  class TC_ActBasicPluginInlcude < Test::Unit::TestCase
    include TestSession

    def assert_day(e, w, user=DEFAULT_USER)
      assert_path(e, w, user, "//div[@class='day']")
    end

    def assert_id_body(e, w, user=DEFAULT_USER)
      assert_path(e, w, user, "//div[@id='body']")
    end

    def assert(e, wikitext)
      assert_id_body(e, wikitext)
    end

    def test_text
      assert_day([[:h2, {:id=>"8f03c3a6dbec1d0f1a5af60947b7b052"}, ""],
		   [:div, {:class=>"body"}, [:div, {:class=>"section"}, []]]],
		 "{{text(* )}}")
      assert_day([[:h2, {:id=>"7657b04993b557a2ee9b36bf280a3ec4"}, ""],
		   [:div, {:class=>"body"}, [:div, {:class=>"section"}, []]]],
		 "{{text\n* \n}}")
      assert_wiki("<p>&lt;</p>", "{{text(<)}}")
      @site["1"].delete
      assert_wiki("<p>&lt;</p>", "{{text\n<\n}}")
      @site["1"].delete
      @site.erase_all

      # test_mail
      assert([:div, {:class=>"day"}, [:h2, {:id=>"From"}, "From:"],
	       [:div, {:class=>"body"},
		 [:div, {:class=>"section"}, [[:p, "a"]]]]],
	     "{{mail\na\n}}")
      assert([:div, {:class=>"day"}, [:h2, {:id=>"From"}, "From:"],
	       [:div, {:class=>"body"},
		 [:div, {:class=>"section"}, [[:p, "eXgłB"]]]]],
	     "* eXg\n{{mail\neXgłB\n\n}}")
      assert([:div, {:class=>"day"},
	       [:h2, {:id=>"979dff15c789fca0315256fc8af6fa64"}, "From: t@e..."],
	       [:div, {:class=>"body"},
		 [:div, {:class=>"section"}, [[:p, "X"]]]]],
	     "* e\n{{mail(t@e.com)\nX\n\n}}")
      assert([:div, {:class=>"day"},
	       [:h2, {:id=>"979dff15c789fca0315256fc8af6fa64"},
		 [:span, {:class=>"date"}, "Jan 1, 1970"], " From: t@e..."],
	       [:div, {:class=>"body"},
		 [:div, {:class=>"section"}, [[:p, "X"]]]]],
	     "* e\n{{mail(t@e.com,0)\nX\n\n}}")
      assert([:div, {:class=>"day"},
	       [:h2, {:id=>"979dff15c789fca0315256fc8af6fa64"}, "From: t@e..."],
	       [:div, {:class=>"body"},
		 [:div, {:class=>"section"}, [[:p, "t@e..."]]]]],
	     "* e\n{{mail(t@e.com)\nt@e.com\n\n}}")

      teardown

      # test_include
      page = @site.create_new
      page.store("* t")
      page2 = @site.create_new
      page2.store("* t2")
      key = page2.key
      assert_day(["",
		   [:div, {:class=>"body"},
		     [:div, {:class=>"section"},
		       [[:div,
			   {:class=>"day"},
			   [:h2, {:id=>"t2"}, "t2"],
			   [:div, {:class=>"body"},
			     [:div, {:class=>"section"}, []]]]]]]],
		 "{{include(#{key})}}")

      assert_id_body([:div,
		       {:class=>"day"},
		       "",
		       [:div,
			 {:class=>"body"},
			 [:div,
			   {:class=>"section"},
			   [[:div,
			       {:class=>"day"},
			       [:h2, {:id=>"t2"}, "t2"],
			       [:div, {:class=>"body"},
				 [:div, {:class=>"section"}, []]]]]]]],
		     "{{include(#{key})}}")

      config = @site["_SiteConfig"]
      config.store(":titlelink:true")
      assert_id_body("<div class=\"day\"><div class=\"body\"><div class=\"section\"><div class=\"day\"><h2><a class=\"label\" href=\"2.html#t2\" name=\"t2\"></a>t2</h2><div class=\"body\"><div class=\"section\"></div><!--section--></div><!--body--></div><!--day--></div><!--section--></div><!--body--></div><!--day-->",
		     "{{include(#{key})}}")

      config.store(":emode_titlelink:true")
      assert_id_body("<div class=\"day\"><div class=\"body\"><div class=\"section\"><div class=\"day\"><h2><a class=\"label\" href=\"2.html#t2\" name=\"t2\"></a>t2</h2><div class=\"body\"><div class=\"section\"></div><!--section--></div><!--body--></div><!--day--></div><!--section--></div><!--body--></div><!--day-->",
		     "{{include(#{key})}}")

      config.store("")

      # test_include_with_titlelink
      t_add_user

      page = @site.create("t1")
      page.store("* t1\n{{include(t2)}}")
      page2 = @site.create("t2")
      page2.store("* t2!")

      session("/test/t1.html")
      assert_text("t1", "title")
      assert_xml("t2!", "//div[@class='day']/h2")

      config = @site["_SiteConfig"]
      config.store(":titlelink:true")
      session("/test/t1.html")
      assert_xml([[:a, {:href=>"t2.html#6b9e49fa28900683969f489ac35161e6",
		       :name=>"6b9e49fa28900683969f489ac35161e6",
		       :class=>"label"}, ""], "t2!"],
		 "//div[@class='day']/h2")

      config = @site["_SiteConfig"]
      config.store(":emode_titlelink:true")
      session("/test/t1.html")
      assert_xml([[:a, {:href=>"t2.html#Xb_M6_lBLMaGGH88jpd-rQ",
		       :name=>"Xb_M6_lBLMaGGH88jpd-rQ",
		       :class=>"label"}, ""], "t2!"],
		 "//div[@class='day']/h2")
    end
  end
end
