# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/generator-editor"

module Qwik
  class Action
    def act_test_editor # only for check design
      w = c_get("TextFormatSimple")
      #w = c_get("TextFormat")
      #w = login_page("http://example.com/HelloQwik/")
      #generate_editor_page("act_test_editor", w, w, w, w)

      title = "Edit | 1"

      pagename = "1"
      str = "t"
      md5hex = str.md5hex

      message = [
	[:div, {:class=>"day"},
	  [:h2, "XVՓ˂܂"],
	  [:div, {:class=>"section"},
	    [:p, "ꂪdiffłB"]
	  ]
	]
      ]
      message = [""]
      require "qwik/act-edit"
      message = save_edit_conflict("1", "differ", "contents")

      edit_form = editor_edit_form(pagename, str, md5hex)

      attach_form = editor_attach_form(pagename)

      main = editor_main(edit_form, attach_form, pagename)

      edit_help = editor_edit_help

      edit_history = editor_edit_history(pagename)

      sidebar = editor_sidebar(edit_help, edit_history)

      generate_editor_page(title, message, main, sidebar)
    end

    def c_get_super(site, k)
      return c_get(k) if site.exist?(k)
      k = "_"+k
      return c_get(k) if site.exist?(k)
      raise "no such super file" # assert
    end

    def c_editor(site, pagename, contents=nil, message=[""])
      page = site[pagename]
      title = _("Edit")+" | "+page.key

      str = contents
      str = page.load if contents.nil?

      md5hex = str.md5hex

      edit_form = editor_edit_form(pagename, str, md5hex)

      attach_form = editor_attach_form(pagename)

      main = editor_main(edit_form, attach_form, pagename)

      edit_help = editor_edit_help

      edit_history = editor_edit_history(pagename)

      sidebar = editor_sidebar(edit_help, edit_history)

      generate_editor_page(title, message, main, sidebar)
    end

    def editor_edit_form(pagename, str, md5hex)
      [:form, {:method=>"POST", :action=>pagename+".save"},
	[:textarea, {:name=>"contents", :id=>"contents",
	    :cols=>"70", :rows=>"20"}, str],
	[:div,
	  [:input, {:type=>"hidden", :name=>"md5hex", :value=>md5hex}],
	  [:input, {:type=>"submit", :name=>"save", :value=>_("Save")}]]]
    end

    def editor_attach_form(pagename)
      [:form, {:action=>pagename+".files",
	  :method=>"POST", :enctype=>"multipart/form-data"},
	[:input, {:type=>"file", :name=>"content", :size=>"50"}],
	[:input, {:type=>"submit", :value=>_("Attach")}]]
    end

    def editor_main(edit_form, attach_form, pagename)
      [
	[:div, {:class=>"day edit"},
	  [:h2, _("Edit")],
	  [:div, {:class=>"section"}, edit_form]],
	[:div, {:class=>"day attach"},
	  [:h2, _("Attach Files")],
	  [:div, {:class=>"section"},
	    attach_form,
	    [:p, [:a, {:href=>pagename+".files"}, _("Attach many files")]]]]
      ]
    end

    def editor_edit_help
      [
	[:ul,
	  [:li, [:tt, "*"], " ", _("Header")],
	  [:li, [:tt, "-"], " ", _("List")],
	  [:li, [:tt, "+"], " ", _("Ordered list")],
	  [:li, [:tt, ">"], " ", _("Blockquote")],
	  [:li, [:tt, ":"], " ", _("Word"), ": ", _("Definition")],
	  [:li, [:tt, ","], " ", _("Table")],
	  [:li, [:tt, "''"], _("Emphasis"), [:tt, "''"]],
	  [:li, [:tt, "'''"], _("Strong"), [:tt, "'''"]],
	  [:li, [:tt, "[["], "FrontPage", [:tt, "]]"], " ", _("Link")]],
	[:p, [:a, {:href=>"TextFormat.html"}, "more help"]]
      ]
    end

    def editor_edit_history(pagename)
      [
	[:ul,
	  [:li, [:a, {:href=>pagename+".backup"}, _("Backup")]],
	  [:li, [:a, {:href=>pagename+".history"}, _("Time machine")]]]
      ]
    end

    def editor_sidebar(edit_help, edit_history)
      [
	[:div, {:class=>"section edithelp"},
	  [:h2, _("Text Format")],
	  *edit_help],
	[:div, {:class=>"section edithistory"},
	  [:h2, _("Edit History")],
	  *edit_history]
      ]
    end

    def generate_editor_page(title, message, main, sidebar)
      @res.status = 200
      template = @memory.template.get("editor")
      @res.body = Qwik::EditorPageGenerator.generate(template, title,
						     message, main, sidebar)
      c_set_html
      c_set_no_cache
      nil
    end
  end
end

if $0 == __FILE__
  require "t/test"
  $debug = true
end

if $debug
  class TC_ActEditor < Test::Unit::TestCase
    include TestSession

    def test_all
      res = session

      @action.generate_editor_page("title", "msg", ["ma"], ["si"])
      assert_equal(200, res.status)
      assert_xpath([:title, "title"], "title", res)
      assert_xpath([:div, {:class=>"message"}, "msg"],
		   "//div[@class='message']")
      assert_xpath([:div, {:class=>"main"}, "ma"],
		   "//div[@class='main']")
      assert_xpath([:div, {:class=>"sidebar"}, "si"],
		   "//div[@class='sidebar']")

      t_add_user

      res = session("/test/.test_editor")
      assert_xpath([:title, "Edit | 1"], "title", res)

      page = @site.create_new
      page.store("t")
      @action.c_editor(@site, "1")
      assert_xpath([:title, "Edit | 1"], "title")
      assert_xpath([:div, {:class=>"message"}, [""]],
		   "//div[@class='message']")

      # edit form
      assert_rattr({"action"=>"1.save", "method"=>"POST"},
		   "//div[@class='day edit']/form")
      assert_xpath([:textarea, {:id=>"contents", :name=>"contents",
		       :cols=>"70", :rows=>"20"}, "t"],
		   "//div[@class='day edit']/form/textarea")
      assert_xpath([:input, {:value=>"e358efa489f58062f10dd7316b65649e",
		       :type=>"hidden", :name=>"md5hex"}],
		   "//div[@class='day edit']/form/input")
      assert_xpath([:input, {:value=>"Save", :type=>"submit", :name=>"save"}],
		   "//div[@class='day edit']/form/input[2]")

      # attach form
      assert_rattr({"enctype"=>"multipart/form-data",
		     "action"=>"1.files", "method"=>"POST"},
		   "//div[@class='day attach']/form")
      assert_xpath([:input, {:size=>"50", :type=>"file", :name=>"content"}],
		   "//div[@class='day attach']/form/input")
      assert_xpath([:input, {:value=>"Attach", :type=>"submit"}],
		   "//div[@class='day attach']/form/input[2]")
      assert_xpath([:p, [:a, {:href=>"1.files"}, "Attach many files"]],
		   "//div[@class='day attach']/p")

      assert_rattr({"href"=>"TextFormat.html"},
		   "//div[@class='sidebar']//a")
    end

    def test_editor
      session

      # test_edit_page_generator
      page = @site.create_new
      page.store("* test\ntestbody\n** h3\nh3body\n* h2\nh2body\n")
      side = @site["_SideMenu"]
      side.store("* side\nsidebody [[1]]")

      session
      @action.c_editor(@site, "1", "contents", "msg")
      assert_text("Edit | 1", "title")
      assert_text("Edit | 1", "h1")
      assert_attr({"name"=>"ROBOTS", "content"=>"NOINDEX,NOFOLLOW"}, "meta")
      assert_in(["msg"], "//div[@class='message']")
      assert_attr({"action"=>"1.save", "method"=>"POST"},
		  "form")
      assert_text("contents", "textarea")
      assert_attr({"name"=>"md5hex", "type"=>"hidden",
		    "value"=>"98bf7d8c15784f0a3d63204441e1e2aa"}, "input")
      assert_attr({"name"=>"save", "type"=>"submit", "value"=>"Save"},
		  "//input[2]")
      assert_rattr({"href"=>"TextFormat.html"},
		   "//div[@class='sidebar']//a")
    end
  end
end
