# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/wabisabi-generator"

module Qwik
  class Action
    def plg_tableform(dest=nil)
      str = yield
      tf = TableForm.new(self, dest, str, @req.base)
      return tf.xml
    end

    def plg_form(dest=nil, m=nil)
      str = yield
      dest = @req.base unless dest && dest != ""
      action = dest.to_s.escape
      action += ".html" if action[0] != ?.
      method = nil
      if m
	method = "GET" if m == "GET"
	method = "POST" if m == "POST"
      end
      hash = {:method=>method, :action=>action}
      hash.delete_if {|k, v| v.nil? }
      return [:form, hash, plg_text(str)]
    end

    def plg_input(name="", value=nil, size=nil, maxsize=nil)
      hash = {:name=>name, :value=>value, :size=>size, :maxsize=>maxsize}
      hash.delete_if {|k, v| v.nil? }
      [:input, hash]
    end

    def plg_input_password(n, v=nil)
      hash = {:type=>"password", :name=>n, :value=>v}
      hash.delete_if {|k, v| v.nil? }
      [:input, hash]
    end

    def plg_submit(v=nil, n=nil)
      hash = {:type=>"submit", :name=>n, :value=>v}
      hash.delete_if {|k, v| v.nil? }
      [:input, hash]
    end

    def plg_textarea(name, cols=nil, rows=nil, msg=nil)
      msg = "" if msg.nil?
      msg = msg.gsub(/\\n/, "\n")
      hash = {:name=>name, :cols=>cols, :rows=>rows}
      hash.delete_if {|k, v| v.nil? }
      [:textarea, hash, msg]
    end
  end

  class TableForm
    def initialize(action, arg, str, pagename=nil)
      @action, @arg, @str, @pagename = action, arg, str, pagename
      @g = HtmlGenerator.new
    end

    def xml
      lines = @str.split(/\n/)
      t = []
      lines.each {|line|
	ar = line.split(/\|/)
	ar.shift
	case ar.length
	when 1
	  t << [:tr, [:td, {:colspan=>2, :class=>"msg"}, ar.shift]]
	when 2
	  midashi = ar.shift
	  td = []
	  nakami = ar.shift
	  nakami.sub!(/^\{\{/, "")
	  nakami.sub!(/\}\}$/, "")
	  if nakami =~ /^([^\(\)]+)\((.*)\)$/
	    cmd, args = $1, $2
	    aa = args.split(/,/)
	    case cmd
	    when "input"
	      td << @action.plg_input(*aa)
	    when "select"
	      td << @g.select(*aa)
	    when "textarea"
	      name, w, h, msg = aa
	      msg = "" if !msg
	      msg.gsub!(/\\n/, "\n")
	      td << [:textarea, {:name=>name, :cols=>w, :rows=>h}, msg]
	    when "submit"
	      su = @g.submit(*aa)
	      su.get_attr["class"] = "submit"
	      td << su
	    when "show"
	      td << [:span, @action.show(*aa)]
	    when "ring_show"
	      td << [:span, @action.plg_ring_show(*aa)]
	    when "member_user"
	      td << [:span, @action.member_user(*aa)]
	    end
	  else
	    td << [:span, nakami]
	  end
	  etd = [:td, {:class=>"nakami"}, td]
	  etr = [:tr, [:td, {:class=>"midashi"}, midashi], etd]
	  t << etr
	else
	  t << [:tr, ar.to_s] #debug
	end
      }
      table = [:table, {:class=>"form"}, t]

      action = @pagename.to_s.escape+".html"
      action = @arg.to_s.escape+".html" if @arg && @arg != ""

      [:div, {:class=>"form"},
	[:form, {:method=>"POST", :action=>action},
	  [:input, {:type=>"hidden", :name=>"page", :value=>@pagename.escape}],
	  table]]
    end
  end
end

if $0 == __FILE__
  $LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
  require "t/test"
  $debug = true
end

if $debug
  class TC_ActForm < Test::Unit::TestCase
    include TestSession

    def test_form
      # test_form_arg
      assert_wiki([:form, {:action=>"DestPage.html"},
		    [[:p, "a"]]],
		  "{{form(DestPage)\na\n}}")
      assert_wiki([:form, {:method=>"POST", :action=>"DestPage.html"},
		    [[:p, "a"]]],
		  "{{form(DestPage, POST)\na\n}}")

      # test_form_element
      assert_wiki([:input, {:name=>"n"}], "{{input(n)}}")
      assert_wiki([:input, {:value=>"v", :name=>"n"}], "{{input(n,v)}}")
      assert_wiki([:input, {:value=>"", :name=>"n"}], "{{input(n,)}}")
      assert_wiki([:input, {:value=>"", :name=>"n"}], "{{input(n,)}}")
      assert_wiki([:input, {:type=>"password", :name=>"n"}],
		  "{{input_password(n)}}")
      assert_wiki([:input, {:value=>"v", :type=>"submit"}], "{{submit(v)}}")
      assert_wiki([:textarea, {:name=>"n"}, ""], "{{textarea(n)}}")
      assert_wiki([:textarea, {:cols=>"40", :name=>"n", :rows=>"7"}, ""],
		  "{{textarea(n,40,7)}}")
      assert_wiki([:textarea, {:cols=>"40", :name=>"n", :rows=>"7"}, "msg"],
		  "{{textarea(n,40,7,msg)}}")
      assert_wiki([:textarea, {:cols=>"40", :name=>"n", :rows=>"7"},
		    "line1nline2"],
		  '{{textarea(n,40,7,line1\nline2)}}') # does not work for now

      # test_format
      assert_wiki([:form, {:action=>"1.html"}, [[:dl, [:dt, "k"], [:dd, "v"]]]],
		  "{{form\n:k:v\n}}")

      # test_real_situation
      assert_wiki([[:dl, [:dt, "dt1"], [:dd, "dd1"]], [:p, "p1"], [:dl, [:dd, "dd2"]]], "{{text\n:dt1:dd1\np1\n::dd2\n}}")
      assert_wiki([[:dl, [:dt, ""], [:dd, [:input, {:name=>"key"}]]],
		    [:p, "L[[h͂ĂB"],
		    [:dl, [:dd,
			[:input, {:value=>"Search", :type=>"submit"}]]]],
		  "{{text
::{{input(key)}}
L[[h͂ĂB
::{{submit( Search )}}
}}")
      assert_wiki([:form, {:action=>"_SiteSearch.html"},
		    [[:dl, [:dt, ""], [:dd, [:input, {:name=>"key"}]]],
		      [:p, "L[[h͂ĂB"],
		      [:dl, [:dd,
			  [:input, {:value=>"Search", :type=>"submit"}]]]]],
		  "{{form(_SiteSearch)
::{{input(key)}}
L[[h͂ĂB
::{{submit( Search )}}
}}")

      teardown

      # test_tableform
      assert_wiki([:div, {:class=>"form"},
		    [:form, {:method=>"POST", :action=>"1.html"},
		      [:input, {:value=>"1", :type=>"hidden", :name=>"page"}],
		      [:table, {:class=>"form"},
			[[:tr,
			    [:td, {:class=>"midashi"}, ""],
			    [:td, {:class=>"nakami"},
			      [[:input, {:size=>"must",
				    :value=>"default", :name=>"name"}]]]],
			  [:tr, [:td, {:class=>"msg", :colspan=>2}, ""]],
			  [:tr,
			    [:td, {:class=>"midashi"}, ""],
			    [:td, {:class=>"nakami"},
			      [[:input, {:value=>" e ",
				    :type=>"submit"}]]]]]]]],
		  "{{tableform
||{{input(name,default,must)}}|
||
||{{submit( e )}}|
}}")
      assert_wiki([:div, {:class=>"form"},
		    [:form, {:method=>"POST", :action=>"dest.html"},
		      [:input, {:value=>"1", :type=>"hidden", :name=>"page"}],
		      [:table, {:class=>"form"},
			[[:tr,
			    [:td, {:class=>"midashi"}, "k"],
			    [:td, {:class=>"nakami"}, [[:span, "v"]]]]]]]],
		  "{{tableform(dest)\n|k|v|\n}}")
      assert_wiki([:div, {:class=>"form"},
		    [:form, {:method=>"POST", :action=>"dest.html"},
		      [:input, {:value=>"1", :type=>"hidden", :name=>"page"}],
		      [:table, {:class=>"form"},
			[[:tr,
			    [:td, {:class=>"midashi"}, "mail"],
			    [:td, {:class=>"nakami"},
			      [[:input, {:size=>"must",
				    :value=>"", :name=>"tomail"}]]]],
			  [:tr, [:td, {:class=>"msg", :colspan=>2},
			      "input your mail."]],
			  [:tr,
			    [:td, {:class=>"midashi"}, ""],
			    [:td, {:class=>"nakami"},
			      [[:input, {:value=>" GO ",
				    :type=>"submit"}]]]]]]]],
		  "{{tableform(dest)
|mail|{{input(tomail,,must)}}|
|input your mail.|
||{{submit( GO )}}|
}}")
    end
  end
end
