# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/parser"
require "qwik/resolve"
require "qwik/generator-view"

module Qwik
  class Action
    #------------------------------------------------------------ format
    def c_parse(str)
      return TextParser.new.parse(str)
    end

    def c_res(str)
      w = c_parse(str)
      return Resolver.new(@config, @site, self).resolve(w)
    end

    def c_get(pagename)
      page = @site[pagename]
      if page.nil?
	return nil
      end
      str = page.load
      return c_res(str)
    end

    def c_add_br(str)
      ignore_chars = [?\s, ?-, ?*, ?>, ?|, ?,, ?{, ?}]
      str.map {|line|
	line.chomp!
	line = line+"~" unless ignore_chars.include?(line[0])
	line
      }.join("\n")+"\n"
    end

    def c_pre_text(str="")
      str = "" if str.nil?
      if block_given?
	str2 = yield
	str += str2 if str2
      end
      str = c_add_br(str)
      c_res(str)
    end

    #------------------------------------------------------------ generate
    def c_set_html
      @res["Content-Type"] = "text/html; charset=Shift_JIS"
    end

    def c_set_no_cache
      @res["Pragma"] = "no-cache"
      @res["Cache-Control"] = "no-cache"
    end

    def c_relative_to_absolute(url)
      return url if %r!\A/! =~ url
      "/"+@site.urlpath+url
    end

    def c_relative_to_full(url)
      if /\Ahttp:\/\// =~ url ||
	  /\Ahttps:\/\// =~ url
	return url # already full
      end

      siteurl = @site.siteconfig["siteurl"]
      if siteurl != ""
	url = siteurl+url
	return url
      end
      url = c_relative_to_absolute(url)
      url = "http://"+@config.public_host+":"+
	((@config.public_port.to_i == 80) ? "" : @config.public_port.to_s)+url
      url
    end

    #------------------------------------------------------------ view
    def c_view(title, &block)
      gen = ViewPageGenerator.new(@config, @memory, self)
      s = ""
      s = block.call if block_given?
      @res.body = gen.generate_mes(@site, title, s)
      c_set_html
      c_set_no_cache
      nil
    end

    def c_wview(title, &block)
      gen = ViewPageGenerator.new(@config, @memory, self)
      s = ""
      s = block.call if block_given?
      @res.body = gen.generate_wmes(@site, title, s)
      c_set_html
      c_set_no_cache
      nil
    end
  end
end

if $0 == __FILE__
  require "t/test"
  $debug = true
end

if $debug
  class TC_ActGenerate < Test::Unit::TestCase
    include TestSession

    def with_site(sitename)
      dir = ("./"+sitename+"/").path
      dir.setup
      yield
      dir.teardown
    end

    def test_c_relative_to_full
      session
      assert_match(/\Ahttp:/,
		   @action.c_relative_to_full("FrontPage.html"))
      assert_equal("http://e.com/",
		   @action.c_relative_to_full("http://e.com/"))
    end

    def test_act_generate
      session

      # Check when siteurl is changed.
      page = @site["_SiteConfig"]
      page.store(":siteurl:http://e.com/")
      assert_equal("http://e.com/FrontPage.html",
		   @action.c_relative_to_full("FrontPage.html"))
      page.delete

      # test_c_relative_to_absolute
      assert_equal("/test/t.html",
		   @action.c_relative_to_absolute("t.html"))
      assert_equal("/test/1.files",
		   @action.c_relative_to_absolute("1.files"))
      @action.instance_eval {
	@site = @memory.farm.get_default_site
      }
      assert_equal("/t.html",
		   @action.c_relative_to_absolute("t.html"))
      assert_equal("/1.files",
		   @action.c_relative_to_absolute("1.files"))
    end

  end
end
