# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

module Qwik
  class Action
    # ============================== icon
    def icon_path(type)
      suf= "gif"
      #suf= "png"
      #path = @config.lib_dir.path+"theme/icons"
      path = @config.theme_dir.path+"icons"
      file = type+"."+suf
      if (path+file).exist?
	return ".theme/icons/"+file
      end
      return nil
    end

    ICON_SUF_TABLE = {
      "tgz" => "tar",
    }

    ICON_MIMETYPE_TABLE = {
      "application/octet-stream" => "binary",
      "application/pdf" => "pdf",
      "application/postscript" => "ps",
      "application/zip" => "compressed",
    }

    ICON_GENRE_TABLE = {
      "application" => "a",
      "audio" => "sound2",
      "image" => "image2",
      "message" => "text",
      "model" => "sphere2",
      "multipart" => "text",
      "text" => "text",
      "video" => "movie",
    }

    def icon_find(ext)
      ext.downcase!

      type = ICON_SUF_TABLE[ext]
      return icon_path(type) if type

      mimetype = @res.mimetypes[ext]
      if mimetype
	type = ICON_MIMETYPE_TABLE[mimetype]
	if type
	  path = icon_path(type)
	  return path if path
	end

	genre, spec = mimetype.split("/")
	type = ICON_GENRE_TABLE[genre]
	if type
	  path = icon_path(type)
	  return path if path
	end
      end

      return icon_path("generic")
    end

    # ============================== thumb
    CONVERT_PATH = "/usr/bin/convert"
    THUMB_SIZE = 50

    def thumb_generate(f)
      org_file = @site.attach.path(f)
      attach_dir = @site.attach.path("")
      thumb_dir = attach_dir+".thumb"
      thumb_dir.check_directory
      thumb = thumb_dir+f

      if ! thumb.exist?
	convert = CONVERT_PATH
	return nil if ! convert.path.exist?
	s = THUMB_SIZE
	geom = "#{s}x#{s}"
	cmd = "#{convert} -size #{geom} #{org_file.to_s} #{thumb.to_s}"
#	qp cmd
	system cmd
      end

      return ".attach/.thumb/"+f.to_s
    end

    # ============================== file thumb
    def file_thumb_generate(f)
      org_file = @site.files(@req.base).path(f)
      files_dir = @site.files(@req.base).path("")
      thumb_dir = files_dir+".thumb"
      thumb_dir.check_directory
      thumb = thumb_dir+f

      if ! thumb.exist?
	convert = CONVERT_PATH
	return nil if ! convert.path.exist?
	s = THUMB_SIZE
	geom = "#{s}x#{s}"
	cmd = "#{convert} -size #{geom} #{org_file.to_s} #{thumb.to_s}"
#	qp cmd
	system cmd
      end

      return @req.base+".files/.thumb/"+f.to_s
    end

    # ============================== plg_file
    def plg_file(f=nil, alt=f)
#      qp f
      if f.nil?
	return
      end

      src = @req.base+".files/"+f

      if ! @site.files(@req.base).exist?(f)
	icon = icon_path("broken")
	return ref_html(src, "icon", icon, alt, alt)
      end

      ext = File.extname(f).sub(/\A\./, "")

      if is_image?(ext)
	thumb = file_thumb_generate(f)
	if thumb # else, go through
	  return ref_html(src, "thumb", thumb, alt, alt)
	end
      end

      icon = icon_find(ext)
      if icon
	return ref_html(src, "icon", icon, alt, alt)
      end

      return [:div, {:class=>"ref"}, 
	[:a, {:href=>src}, alt]]
    end

    # ============================== plg_ref
    def ref_html(href, klass, src, alt, message)
      return [:div, {:class=>"ref"}, 
	[:a, {:href=>href},
	  [:img, {:class=>klass, :src=>src, :alt=>alt}],
	  [:br], message]]
    end

    def is_image?(ext)
      ext.downcase!
      mimetype = @res.mimetypes[ext]
      return false if mimetype.nil?
      genre, spec = mimetype.split("/")
      return genre == "image"
    end

    def plg_ref(f=nil, alt=f)
      if f.nil?
	return [:div, {:class=>"ref"}, attach_form(@req.base+".attach")]
      end

      src = ".attach/"+f

      if ! @site.attach.exist?(f)
	icon = icon_path("broken")
	return ref_html(src, "icon", icon, alt, alt)
      end

      ext = File.extname(f).sub(/\A\./, "")

      if is_image?(ext)
	thumb = thumb_generate(f)
	if thumb # else, go through
	  return ref_html(src, "thumb", thumb, alt, alt)
	end
      end

      icon = icon_find(ext)
      if icon
	return ref_html(src, "icon", icon, alt, alt)
      end

      return [:div, {:class=>"ref"}, 
	[:a, {:href=>src}, alt]]
    end
  end
end

if $0 == __FILE__
  $LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
  require "t/test"
  $debug = true
end

if $debug
  class TC_ActRef < Test::Unit::TestCase
    include TestSession

    def test_all
      # test_find_icon
      session("/")
      assert_equal(".theme/icons/tar.gif", @action.icon_find("tgz"))
      assert_equal(".theme/icons/text.gif", @action.icon_find("txt"))
      assert_equal(".theme/icons/text.gif", @action.icon_find("css"))
      assert_equal(".theme/icons/image2.gif", @action.icon_find("png"))
      assert_equal(".theme/icons/image2.gif", @action.icon_find("gif"))
      assert_equal(".theme/icons/pdf.gif", @action.icon_find("pdf"))
      assert_equal(".theme/icons/ps.gif", @action.icon_find("ps"))
      assert_equal(".theme/icons/compressed.gif", @action.icon_find("zip"))
      assert_equal(".theme/icons/binary.gif", @action.icon_find("exe"))
      assert_equal(".theme/icons/generic.gif", @action.icon_find("nosuchext"))

      # test is_image?
      assert_equal(true, @action.is_image?("jpg"))
      assert_equal(true, @action.is_image?("jpeg"))
      assert_equal(true, @action.is_image?("JPG"))
      assert_equal(true, @action.is_image?("png"))
      assert_equal(true, @action.is_image?("gif"))
      assert_equal(true, @action.is_image?("bmp"))
      assert_equal(true, @action.is_image?("ico"))
      assert_equal(true, @action.is_image?("ppm"))
      assert_equal(false, @action.is_image?("pdf"))

      # test_ref_plugin
      @site.attach.delete("1x1.png") if @site.attach.exist?("1x1.png")

      assert_wiki([:div, {:class=>"ref"},
		    [:form, {:method=>"post", :action=>"1.attach", :enctype=>"multipart/form-data"},
		      [:input, {:style=>"width:30em;",
			  :type=>"file", :name=>"content"}],
		      [:br],
		      [:input, {:value=>"attach", :type=>"submit"}]]],
		  "{{ref}}")

      assert_wiki([:div, {:class=>"ref"},
		    [:a, {:href=>".attach/1x1.png"},
		      [:img, {:alt=>"1x1.png", :class=>"icon",
			  :src=>".theme/icons/broken.gif"}],
		      [:br], "1x1.png"]],
		  "{{ref(1x1.png)}}")

      png = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000\001\000\000\000\001\010\002\000\000\000\220wS\336\000\000\000\fIDATx\332c\370\377\377?\000\005\376\002\3763\022\225\024\000\000\000\000IEND\256B`\202"
      @site.attach.put("1x1.png", png)
      assert_wiki([:div, {:class=>"ref"},
		    [:a, {:href=>".attach/1x1.png"},
		      [:img, {:alt=>"1x1.png", :class=>"thumb",
			  :src=>".attach/.thumb/1x1.png"}],
		      [:br], "1x1.png"]],
		  "{{ref(1x1.png)}}")

      @site.attach.delete("test.pdf") if @site.attach.exist?("test.pdf")
      pdf = "PDF...dummy"
      @site.attach.put("test.pdf", pdf)
      assert_wiki([:div, {:class=>"ref"},
		    [:a, {:href=>".attach/test.pdf"},
		      [:img, {:alt=>"test.pdf", :class=>"icon", :src=>".theme/icons/pdf.gif"}],
		      [:br], "test.pdf"]],
		  "{{ref(test.pdf)}}")

      @site.attach.delete("test.txt") if @site.attach.exist?("test.txt")
      file = "text file dummy"
      @site.attach.put("test.txt", file)
      assert_wiki([:div, {:class=>"ref"},
		    [:a, {:href=>".attach/test.txt"},
		      [:img, {:alt=>"test.txt", :class=>"icon", :src=>".theme/icons/text.gif"}],
		      [:br], "test.txt"]],
		  "{{ref(test.txt)}}")

      if @site.attach.exist?("test.nosuchext")
	@site.attach.delete("test.nosuchext")
      end
      file = "unknown file dummy"
      @site.attach.put("test.nosuchext", file)
      assert_wiki([:div, {:class=>"ref"},
		    [:a, {:href=>".attach/test.nosuchext"},
		      [:img, {:alt=>"test.nosuchext", :class=>"icon", 
			  :src=>".theme/icons/generic.gif"}],
		      [:br], "test.nosuchext"]],
		  "{{ref(test.nosuchext)}}")
    end
  end
end
