# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

module Qwik
  class EmodePreProcessor # special mode, not for public use
    def preprocess(sar)
      hmark = {"h2"=>"*", "h3"=>"**"}
      next_line = nil
      num = 0
      ar = []
      sar.each {|line|
	line.chomp!
	if next_line
	  if next_line == "h2"
	    num += 1
	  end
	  if /\A([])(.*)\z/s =~ line
	    ar << hmark[next_line]+$2
	  else
	    if next_line == "h2"
	      ar << "*"+num.to_s
	    else
	      ar << "===="
	    end
	    next_line = add_line(next_line, ar, line)
	  end
	  next_line = nil
	else
	  next_line = add_line(next_line, ar, line)
	end
      }
      ar
    end

    def add_line(next_line, ar, line)
      case line
      when /\A(=+)\z/ # only =
	return "h2"
      when /\A(------+)/ # only -
	return "h3"
      when /\A([E])(.*)\z/s
	ar << "-"+$2
	return next_line
      when /\A([])(.*)\z/s
	ar << "***"+$2
	return next_line
      end

#     if /\A([\ \-\*\>])(.*)\z/s =~ line
      ignore_chars = [?\s, ?-, ?*, ?>, ?|, ?,, ?{, ?}]
      if ignore_chars.include?(line[0]) ||
	  line == ""
	ar << line
	return next_line
      end

      ar << line+"{{br}}"
      return next_line
    end
  end
end

$debug = true if $0 == __FILE__
if $debug
  require "test/unit"

  class TC_EmodePreProcessor < Test::Unit::TestCase
    def assert(e, str)
      assert_equal(e, Qwik::EmodePreProcessor.new.preprocess("============================================================\n"+str).join("\n"))
    end

    def test_all
      assert("*t", "t")
      assert("*1\n-t", "Et")
      assert("*1\nt{{br}}", "t")
      assert("*t\nt1{{br}}\nt2{{br}}", "t\nt1\nt2")
      assert("*t\nt1{{br}}\n====\nt2{{br}}",
	     "t\nt1\n------------------------------------------------------------\nt2")
      #assert("*t\n>t1{{br}}\n>t2{{br}}", "t\n>t1\n>t2") # impossible...
      assert("*t\n>t1\n>t2", "t\n>t1\n>t2")
      assert("*1\n-", "-")
      assert("*1",
	     "------------------------------------------------------------")

      # test_sjis_bug
      assert("*1\n[{{br}}\nt{{br}}", "[\nt")
    end

    def test_br
      assert("*1\n{{{\nt{{br}}\n}}}", "{{{\nt\n}}}")
    end

    def test_multiline
      assert("*1\na{{br}}\n\nb{{br}}", "a\n\nb")
    end
  end
end
