# copied from gonzui-0.1

#
# gonzui - a source code search engine.
#
# Copyright (C) 2004 Satoru Takabayashi <satoru@namazu.org> 
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the GNU General Public License version 2.
#

#module Gonzui
module Qwik
  module GetText
    def gettext (text)
      return text if !defined?(@gettext_catalog) || @gettext_catalog.nil?
      return (@gettext_catalog[text] or text)
    end
    alias :_ :gettext

    def load_catalog (file_name)
      return eval(File.read(file_name))
    end

    def load_catalogs (directory)
      catalogs = Hash.new
      Dir.entries(directory).each {|entry|
        file_name = File.join(directory, entry)
        if m = /^catalog\.([\w.-]+)$/.match(File.basename(file_name))
          lang = m[1]
          catalog = load_catalog(file_name)
          catalogs[lang] = catalog
        end
      }
      catalogs["en"] = Hash.new {|h, k| k }
      return catalogs
    end

    def set_catalog (catalog)
      @gettext_catalog = catalog
    end
  end
end

if $0 == __FILE__
  require "test/unit"
  $debug = true
end

if $debug
  class TC_GetText < Test::Unit::TestCase
    include Qwik::GetText

    def setup
      @catalog_file_name = "catalog.ja"
      @catalog = {
	"hello" => "ɂ"
      }
    end

    def make_catalog
      File.open(@catalog_file_name, "w") {|f|
	f.puts "{"
	@catalog.each {|key, value|
	  f.printf('  "%s" => "%s",', key, value)
	  f.puts
	}
	f.puts "}"
      }
    end

    def remove_catalog
      File.unlink(@catalog_file_name)
    end

    def test_gettext
      make_catalog

      catalog = load_catalog(@catalog_file_name)
      assert_equal(catalog, @catalog)

      assert_equal("hello", _("hello"))
      set_catalog(catalog)
      assert_equal("ɂ", _("hello"))

      set_catalog({})
      assert_equal("hello", _("hello"))
      catalogs = load_catalogs(".")
      set_catalog(catalogs["ja"])
      assert_equal("ɂ", _("hello"))

      remove_catalog
    end
  end
end
