# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

$LOAD_PATH.unshift("../../compat") unless $LOAD_PATH.include?("../../compat")
require "htree"
$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/htree-get"

module Qwik
  class HtmlToWikiText
    def translate(text)
      h = HTree(text)
      parse_block(h)
    end

    def html_to_wabisabi(str)
      html = "<html>"+str+"</html>"
      htree = HTree(html)
      first_child = htree.children[0]
      wabisabi = first_child.to_wabisabi       #pp wabisabi
      wabisabi_child = wabisabi.inside         #pp wabisabi_child
      v = wabisabi_child.check_valid
      if v == true
	return wabisabi_child
      else
	return "can not use ["+v.to_s+"]"
      end
    end

    private

    def parse_block(elem)
      str = ""
      elem.each_child {|e|
	case e
	when HTree::Text
	  t = e.to_s
	  next if t == "\r\n" || t == "\n"
	  qp "What? "+e.class.to_s+" : "+e.to_s
	when HTree::Elem
	  str << parse_block_elem(e)
	else
	  qp "What? "+e.class.to_s+" : "+e.to_s
	end
      }
      str
    end

    SIMPLE_BLOCK = {
      "H2"=>["* ", "\n"],
      "H3"=>["** ", "\n"],
      "H4"=>["*** ", "\n"],
      "H5"=>["**** ", "\n"],
      "H6"=>["***** ", "\n"],
      "P"=>["", "\n\n"],
    }

    def parse_block_elem(e)
      n = e.element_name.to_s

      if SIMPLE_BLOCK[n]
	head, tail = SIMPLE_BLOCK[n]
	return head+parse_span(e)+tail
      end

      case n
      when "PRE"
	return parse_pre(e)
      when "BLOCKQUOTE"
	return parse_blockquote(e)
      when "PLUGIN"
	attr = e.attributes_str
	param = attr["param"]
	method = attr["method"]
	#data = e.text
	data = e.extract_text.to_s
	str = ""
	str << "{{"+method
	str << "("+param+")" if param && param != ""
	str << "\n"+data+"\n" if data && data != ""
	str << "}}\n"
	return str
      else
#	qp "What? "+e.class.to_s+" : "+e.to_s
	return ""
      end
    end

    def parse_pre(elem)
      str = ""
      elem.each_child {|e|
	str << " "+e.to_s+"\n"
      }
      str
    end

    def parse_blockquote(elem)
      str = parse_block(elem)
      str.map {|line|
	"> "+line
      }.join
    end

    def parse_span(elem)
      str = ""
      elem.each_child {|e|
	case e
	when HTree::Text
	  str << e.to_s
	when HTree::Elem
	  str << parse_span_elem(e)
	else
	  qp "What? "+e.class.to_s+" : "+e.to_s
	end
      }
      str
    end

    def parse_span_elem(e)
      n = e.element_name.to_s
      case n
      when "A"
	attr = e.attributes_str
	href = attr["href"]
	if href =~ /\A([A-Za-z0-9]+)\.html\z/
	  href = $1
	end
	#text = e.text
	text = e.extract_text.to_s
	str = ""
	str << "[["+text
	str << "|"+href if href != text
	str << "]]"
	return str
      when "IMG"
	attr = e.attributes_str
	src = attr["src"]
	return "[["+src+"]]"
      when "EM"
	return "''"+parse_span(e)+"''"
      when "STRONG"
	return "'''"+parse_span(e)+"'''"
      when "DEL"
	return "=="+parse_span(e)+"=="
      else
	qp "What? "+e.class.to_s+" : "+e.to_s
	return ""
      end
    end

  end
end

if $0 == __FILE__
  $LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
  require "test/unit"
  $debug = true
end

if $debug
  class TC_HtmlToWikiText < Test::Unit::TestCase
    def assert(e, text)
      trans = Qwik::HtmlToWikiText.new
      str = trans.translate(text)
      assert_equal(e, str)
    end

    def test_html_to_wikitext
      # test div
      assert("* h\n", "<H2>h</H2>")
      assert("** h\n", "<H3>h</H3>")
      assert("p\n\n", "<P>p</P>")
      assert("p\n\np2\n\n", "<P>p</P><P>p2</P>")
      #assert("", "<UL><LI>li</LI></UL>")
      #assert("", "<UL><LI>li1</LI><UL><LI>li2</LI></UL></UL>")
      #assert("", "<DL><DT>dt<DD>dd</DD></DL>")
      #assert("", "<DL><DT>dt1<DD>dd1><DT>dt2<DD>dd2</DD></DL>")
      assert("> pB\n> \n", "<BLOCKQUOTE><P>pB</P></BLOCKQUOTE>")
      assert("> * h\n", "<BLOCKQUOTE><H2>h</H2></BLOCKQUOTE>")
      assert("> x\n> \n> y\n> \n", "<BLOCKQUOTE><P>x</P><P>y</P></BLOCKQUOTE>")

      # test span
      assert("Go [[FrontPage]].\n\n",
	     '<P>Go <A href="FrontPage.html">FrontPage</A>.</P>')
      assert("Go [[qwikWeb|http://example.com/]].\n\n", '<P>Go <A href="http://example.com/">qwikWeb</A>.</P>')
      assert("''''A'''ɋ'''A====\n\n", '<P><EM></EM>A<STRONG>ɋ</STRONG>A<DEL></DEL></P>')
      assert("[[http://example.com/.theme/new.png]]\n\n", '<P><IMG alt=new src="http://example.com/.theme/new.png"></P>')
      assert("[[FrontPage]] [[Yahoo!|http://www.yahoo.co.jp/]]\n\n{{recent(1)}}\n", '<P><A href="FrontPage.html">FrontPage</A> <A href="http://www.yahoo.co.jp/">Yahoo!</A></P><PLUGIN param="1" method="recent"></PLUGIN>')

      s = <<'EOT'
<H2>FrontPage</H2>
<P>͐VKqwikWebTCg̓ƂȂy[WłB</P>
<H3>g</H3>
<P>y[W̏̕ɂuҏWvƂNǂƁÃy[W̕ҏW[hɂȂ܂B</P>
<P>\ꂽeLXg̓eύXAuSavev{^NbNƁÃy[W̓eύX܂B</P>
<H3>Lq@</H3>
<P>y[W̓e̓eLXgŏĂA̋LɂČoȂǂ̎w܂Bڂ́A<A href="TextFormat.html">TextFormat</A>B</P>
<H3>qwikWeb</H3>
<P>ڂ́A<A href="http://example.com/">qwikWeb</A>z[y[WB</P>
EOT

      org = <<'EOT'
* FrontPage
͐VKqwikWebTCg̓ƂȂy[WłB

** g
y[W̏̕ɂuҏWvƂNǂƁA
̃y[W̕ҏW[hɂȂ܂B

\ꂽeLXg̓eύXAuSavev{^NbNƁA
̃y[W̓eύX܂B

** Lq@
y[W̓e̓eLXgŏĂA
̋LɂČoȂǂ̎w܂B
ڂ́A[[TextFormat]]B

** qwikWeb
ڂ́A[[qwikWeb|http://example.com/]]z[y[WB
EOT

      e = <<'EOT'
* FrontPage
͐VKqwikWebTCg̓ƂȂy[WłB

** g
y[W̏̕ɂuҏWvƂNǂƁÃy[W̕ҏW[hɂȂ܂B

\ꂽeLXg̓eύXAuSavev{^NbNƁÃy[W̓eύX܂B

** Lq@
y[W̓e̓eLXgŏĂA̋LɂČoȂǂ̎w܂Bڂ́A[[TextFormat]]B

** qwikWeb
ڂ́A[[qwikWeb|http://example.com/]]z[y[WB

EOT

      assert(e, s)

      s = <<'EOT'
<H2>ꗗȈՔ</H2>
<P>ڍׂȐ<A href="TextFormat.html">TextFormat</A>񂭂B</P>
<H3>o2</H3>
<H4>o3</H4>
<H5>o4</H5>
<H6>o5</H6>
<UL>
<LI>ӏx1
<UL>
<LI>ӏx2
<UL>
<LI>ӏx3</LI></UL></LI></UL></LI></UL>
<OL>
<LI>Xg1
<OL>
<LI>Xg2
<OL>
<LI>Xg3</LI></OL></LI></OL></LI></OL><PRE>`ς݃eLXgB</PRE>
<BLOCKQUOTE>
<P>pB</P></BLOCKQUOTE>
<DL>
<DT>Wiki
<DD>݉\Weby[W
<DT>QuickML
<DD>ȒPɍ郁[OXgVXe</DD></DL>
<TABLE>
<TBODY>
<TR>
<TD>1-1</TD>
<TD>1-2</TD>
<TD>1-3</TD></TR>
<TR>
<TD>2-1</TD>
<TD>2-2</TD>
<TD>2-3</TD></TR></TBODY></TABLE>
<P><EM></EM>A<STRONG>ɋ</STRONG>A<DEL></DEL> <IMG alt=new src="http://example.com/.theme/new.png"> <A href="FrontPage.html">FrontPage</A> <A href="http://www.yahoo.co.jp/">Yahoo!</A></P><PLUGIN param="1" method="recent"></PLUGIN>
EOT

      org = <<'EOT'
* ꗗȈՔ
ڍׂȐ[[TextFormat]]񂭂B
** o2
*** o3
**** o4
***** o5
- ӏx1
-- ӏx2
--- ӏx3
+ Xg1
++ Xg2
+++ Xg3
 `ς݃eLXgB
> pB
:Wiki:݉\Weby[W
:QuickML:ȒPɍ郁[OXgVXe
,1-1,1-2,1-3
,2-1,2-2,2-3
''''A'''ɋ'''A====
[[new|http://example.com/.theme/new.png]]
[[FrontPage]]
[[Yahoo!|http://www.yahoo.co.jp/]]
{{recent(1)}}
EOT

      e = <<'EOT'
* ꗗȈՔ
ڍׂȐ[[TextFormat]]񂭂B

** o2
*** o3
**** o4
***** o5
 `ς݃eLXgB
> pB
> 
''''A'''ɋ'''A==== [[http://example.com/.theme/new.png]]

{{recent(1)}}
EOT

      assert(e, s)

#    page = @site.create_new
#    str = "{{show_history}}"
#    page.store(str)
    end
  end
end
