# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.
# This file is in Shift-JIS character encoding.  Japanese Character -> ""

$LOAD_PATH.unshift("../../compat") unless $LOAD_PATH.include?("../../compat")
require "base64"
$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/util-kconv"
require "qwik/attach"
require "qwik/config"
require "qwik/memory"
require "qwik/farm"
require "qwik/site"
require "qwik/wikidb"

$debug = true if $0 == __FILE__
if $debug
  require "quickml/quickml"
end

module QuickML
  $qwik_dir = File.expand_path(File.dirname(__FILE__)+"/../..")

  class Config
  end

  class QwikSite
    def initialize(sitename, test=false)

      if test
	@config = $test_config
	@memory = $test_memory
      else
	@config = Qwik::Config.new(test)
	@memory = Qwik::ServerMemory.new(@config)
      end

      @sitename = sitename
      @site = @memory.farm.get_site(@sitename)
      @key = nil
    end
    attr_reader :key

    def valid_as_pagekey?(t) # same as in act-newpage.rb
      /^[A-Za-z_0-9]+$/ =~ t
    end

    def post(mail)
      # get subject
      s = mail.get_clean_subject

      page = @site.get_by_title(s)
      if page.nil?
	# CREATE
	page = valid_as_pagekey?(s) ? @site.create(s) : @site.create_new
	page.store("* "+s+"\n")
      end
      @key = page.key # @key will be used at qwik_url

      # get body
      message = make_message(mail)
      page.add(message)
#      @site.close
    end

    private

    def make_message(mail)
      from = mail.mail_from
      body = ""
      mail.each_part {|mail|
	content_type = mail.content_type
	if mail.plain_text_body?
	  body << mail.body.chomp.chomp+"\n"
	else
	  filename = mail.filename
	  content = mail.decoded_body
	  if filename && content
	    msg = attach(filename, content)
	    body << msg
	  end
	end
      }

      t = Time.now
      t = Time.at(0) if $quickml_debug
      time = t.to_i.to_s
      message = "{{mail(#{from},#{time})\n#{body}}}\n".jistosjis
      message
    end

    def attach(filename, content)
      if @site.attach.exist?(filename)
	return "Site attach failed because the same filename is already used.\n"
      end
      @site.attach.put(filename, content)
      "\n{{ref(#{filename})}}\n\n"
    end
  end

  class << QuickML
    remove_method :valid_name?
    def valid_name? (name)
      # do not allow "_" and "." for short_name
      /^([0-9a-zA-Z-]+)(@[0-9a-zA-Z_.-]+)?$/ =~ name
    end
  end

  class QuickML
    alias org_initialize initialize
    def initialize (config, address, creator = nil, message_charset = nil)
      @config = config
      @address = address
      @name = get_name(@address)
      raise InvalidMLName unless QuickML.valid_name?(@name)
      @short_name = @name.split("@").first

      @return_address = generate_return_address

      init_files
      @member_added_p = false
      @added_members = []

      @logger = @config.logger
      @catalog = @config.catalog

      init_ml_config
      init_members
      init_count
      init_charset

      @message_charset = (message_charset or @charset)
      @logger.log "[#{@name}]: New ML by #{creator}" if newly_created?
    end

    private
    def init_files
      # original QuickML
      #name = @name
      #dir = @config.data_dir
      
      # for qwikWeb
      name = ""
      dir = File.join(@config.data_dir, @name)
      Dir.mkdir(dir) unless FileTest.directory?(dir)

      @members_file = File.join(dir, name + ",members")
      @count_file = File.join(dir, name + ",count")
      @forwardp_file = File.join(dir, name + ",forward")
      @alertedp_file = File.join(dir, name + ",alerted")
      @permanentp_file = File.join(dir, name + ",permanent")
      @unlimitedp_file = File.join(dir, name + ",unlimited")
      @charset_file = File.join(dir, name + ",charset")
      @ml_config_file = File.join(dir, name + ",config")
      @waiting_members_file = File.join(dir, name + ",waiting-members")
      @waiting_message_file = File.join(dir, name + ",waiting-message")
    end

    undef generate_footer
    def generate_footer (member_list_p = false)
      footer = "\n-- \n" + _("archive-> %s ", qwik_url) + "\nML-> #{@address}\n"
      footer << unsubscribe_info if member_added?
      footer << "\n" + member_list if member_added? or member_list_p
      footer
    end

    def qwik_url
      base = ""
      base = @key+".html" if defined?(@key) && @key
      "http://"+@config.domain+"/"+@name+"/"+base
    end

    undef last_article_time
    def last_article_time # the mtime of the newest file in the directory
      max = maxfile = nil
      dir = File.dirname(@count_file)
      Dir.foreach(dir){|f|
	next if f =~ /^\./ || f =~ /,config/ || f =~ /,charset/
	file = File.join(dir, f)
	mt = File.mtime(file)
	if max.nil? || max < mt
	  max = mt 
	  maxfile = file
	end
      }
      max || Time.now
    end

    def qwik_post(mail, test=false)
      site = QwikSite.new(self.name, test)
      site.post(mail)
      @key = site.key # @key will be used at qwik_url
      @logger.log "[#{@name}]: QwikPost: #{@key}"
    end

    if !defined? _org_submit
      alias _org_submit _submit
      def _submit(mail)
	qwik_post(mail)
	_org_submit(mail)
      end
    end

    undef send_confirmation
    def send_confirmation (creator_address)
      header = []
      subject = Mail.encode_field(_("[%s] Confirmation: %s",
				    @short_name, @address))
      header.push(["To",	creator_address],
		  ["From",	confirmation_address],
		  ["Subject",	subject],
                  ["Content-Type", content_type])

      body = confirmation_message(@address)
      Mail.send_mail(@config.smtp_host, @config.smtp_port, @logger, 
		     :mail_from => "", 
		     :recipient => creator_address,
		     :header => header,
		     :body => body)
      @logger.log "[#{@name}]: Send confirmation: #{confirmation_address} #{creator_address}"
    end

    def confirmation_message(address)
      body = ""
      body += _("At the first, please read the agreement of this service.\n")
      body += _("http://example.com/qwikjpAgreementE.html\n")
      body += _("You have to agree this agreement to use this service.\n")
      body += _("If you agree, then,\n")
      body += _("Please simply reply this mail to create ML <%s>.\n",
               address)
      body
    end
  end

  class Processor
    undef report_invalid_mladdress
    def report_invalid_mladdress (mladdress)
      header = []
      subject = Mail.encode_field(_("[QuickML] Error: %s", @mail["Subject"]))
      header.push(["To",	@mail.from],
		  ["From",	@config.postmaster],
		  ["Subject",	subject],
                  ["Content-type", content_type])

      body =   _("Invalid mailing list name: <%s>\n", mladdress)
      body <<  _("You can only use 0-9, a-z, A-Z,  `-' for mailing list name\n")

      body << generate_footer
      Mail.send_mail(@config.smtp_host, @config.smtp_port, @logger,
		     :mail_from => '', 
		     :recipient => @mail.from,
		     :header => header,
		     :body => body)
      @logger.log "Invalid ML Address: #{mladdress}"
    end
  end

  class Mail
    undef parts
    def parts
      parts = @body.split(/^--#{Regexp.escape(self.boundary)}-*\n/)
      parts.shift  # Remove the first empty string.
      parts
    end

    def decoded_body
      encoding = self["Content-Transfer-Encoding"]
      return Base64.decode64(@body) if encoding == "base64"
      if encoding == "x-uuencode"
	uu = ""
	@body.each_line{|line|
	  next if /\Abegin/ =~ line
	  break if /\Aend/ =~ line
	  uu << line
	}
	return uu.unpack("u").first
      end
      @body
    end

    def filename
      disp = self["Content-Disposition"]
      return nil if disp.nil?
      if /\Aattachment;\s+filename=\"(.*)\"\Z/ =~ disp # should use \z ?
	filename = $1.jistosjis
	return filename
      end
      nil
    end

    # 肳Ή[hpӂׂƁB

    def get_clean_subject
      str = Mail.clean_subject(self["Subject"])
      str.sub!(/(?:Re:\s*)+/i, "")
      str.sub!(/^\s+/, "")
      str.sub!(/\s+$/, "")
      str
    end

    def plain_text_body?
      self["Content-Type"] == "" or
	%r!\btext/plain\b!i =~ self["Content-Type"] #/
    end

    def get_body
      body = ""
      self.each_part {|mail|
	content_type = mail.content_type
	if mail.plain_text_body?
	  body << mail.body.chomp.chomp+"\n"
	end
      }
      body
    end

    def each_part(&block)
      if multipart?
	self.parts.each {|str|
	  submail = Mail.new
	  submail.read(str)
	  submail.each_part(&block) # recursive
	}
      else
	yield(self)
      end
    end
  end

  class Sweeper
    private
    undef ml_file?
    def ml_file? (filename)
      return false if File.file?(filename)
      return false if File.basename(filename) =~ /\./ # avoid the name with dot
      true
    end
  end
end
