# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

$LOAD_PATH.unshift("../../compat") unless $LOAD_PATH.include?("../../compat")
require "base64"
$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/tdiary-resolver"

module Qwik
  class Resolver
    def initialize(config, site, action)
      @config = config
      @site = site
      @action = action
    end

    def resolve(wabisabi)
      wabisabi.each_tag(:a, :plugin){|w|
	# at the first, resolve plugin
	if w.first == :plugin
	  w = resolve_plugin(w)
	end

	# then resolve a link.
	if w.first == :a
	  ww = resolve_ref(w)
	  w = ww ? ww : [w]
	end
	w
      }
    end

    def resolve_plugin(w)
      attr = w[1]
      return [] if attr.nil? || !attr.is_a?(Hash)
      method = attr[:method].to_s
      param  = attr[:param].to_s
      data   = w[2].to_s
      result = @action.c_call_plugin(method, param, data)
      result = [] if result.nil?
      result = [result] if ! result.is_a?(Array)
      result
    end

    def set_attr(w, attr)
      w[1].update(attr)
    end

    def resolve_ref(w)
      attr = w[1]
      return nil if attr.nil? || !attr.is_a?(Hash)
      href = attr[:href].to_s

      if /^(?:http|https|ftp|file):\/\// =~ href	# external link
	set_attr(w, {:class=>"external"})
#	set_attr(w, {:title=>"hoge"})
#	set_attr(w, {:"ap:banner"=>"hoge"})
	if @site.external_site? || # do not convert if external site 
	    @site.is_open? || # do not convert if open site
	    @site.siteconfig["redirect"] != "true"
	  return [w]
	end
	href = "/"+href
	set_attr(w, {:href=>href})
	return [w]
      end
      return nil if /\?/ =~ href	# ignore command link
      return nil if /\.html\z/ !~ href	# ignore not html file
      return nil if href[0] == ?/	# already resolved

      pagename = href.sub(/\.html\z/, "")
      text = w[2]

      t = pagename.unescape.to_s
      thepage = @site.get_by_title(t)
      nt = thepage.key.to_s if thepage
      if nt && t != nt
	pagename = thepage.key.to_s
	set_attr(w, {:href=>pagename+".html"})
	return [w]
      end

      if !@site.exist?(pagename) # create a new page link
	title = pagename.unescape
	newpagename = title.escape
	#edithref = "#{newpagename}.new"
	edithref = ".new?t=#{newpagename}"
	return [[:span, {:class=>"new"}, text],
	  [:a, {:class=>"new", :href=>edithref},
	    [:img, {:src=>"/.theme/i/new.png", :alt=>"create", :class=>"new"}]]]
      end

      if pagename == text
	t = @site[pagename].get_title
	return [w] if t == pagename
	w[2] = t
	return [w]
      end
      return [w]
    end
  end
end

if $0 == __FILE__
  require "t/test"
  $debug = true
end

if $debug
  class TC_Resolver < Test::Unit::TestCase
    include TestSession

    def assert(e, wabisabi)
      res = Qwik::Resolver.new(@config, @site, @action)
      w = res.resolve(wabisabi)
      assert_equal(e, w)
    end

    def test_all
      session

      # test_plugin
      assert([[:br]], [[:plugin, {:method=>"br", :param=>""}, ""]])
      assert(["nosuch plugin"],
	     [[:plugin, {:method=>"nosuchplugin", :param=>""}, ""]])

      # test_interwiki
      assert([[:span, {"class"=>"interwiki"}, "Test:t"]],
	     [[:plugin, {:method=>"interwiki", :param=>"Test:t"}, ""]])
      page = @site.create("_InterWikiName")
      page.store(",Test,http://example.com/?q=,sjis")
      assert([[:a, {"href"=>"http://example.com/?q=t",
		   "class"=>"interwiki"}, "Test:t"]],
	     [[:plugin, {:method=>"interwiki", :param=>"Test:t"}, ""]])

      # test_interwiki_error
      assert([],
	     [[:plugin, {:method=>"interwiki", :param=>'"H_R", "h://e.c/"'},
		 ""]])

      # test_ref
      assert([[:span, {:class=>"new"}, "t"],
	       [:a, {:class=>"new", :href=>".new?t=t"},
		 [:img, {:alt=>"create", :src=>"/.theme/i/new.png",
		     :class=>"new"}]]], [[:a, {:href=>"t.html"}, "t"]])
      page = @site.create("t")
      assert([[:a, {:href=>"t.html"}, "t"]],
	     [[:a, {:href=>"t.html"}, "t"]])
      page = @site.create_new
      page.store("t")
      assert([[:a, {:href=>"1.html"}, "1"]],
	     [[:a, {:href=>"1.html"}, "1"]])
      page.store("*t")
      assert([[:a, {:href=>"1.html"}, "t"]],
	     [[:a, {:href=>"1.html"}, "1"]])
      assert([[:a, {:href=>".attach"}, ".attach"]],
	     [[:a, {:href=>".attach"}, ".attach"]])
    end
  end

  class TC_ResolverPlugin < Test::Unit::TestCase
    include TestSession

    def assert(e, wabisabi)
      res = Qwik::Resolver.new(@config, @site, @action)
      w = res.resolve_plugin(wabisabi)
      assert_equal(e, w)
    end

    def test_all
      session

      # test_resolve_plugin
      assert([], [:a])
      assert([], [[:plugin]]) # error
      assert([], [[:plugin, "t"]]) # error
      assert([:br], [:plugin, {:method=>"br", :param=>""}, ""])
    end
  end

  class TC_ResolverRef < Test::Unit::TestCase
    include TestSession

    def assert(e, wabisabi)
      res = Qwik::Resolver.new(@config, @site, @action)
      w = res.resolve_ref(wabisabi)
      assert_equal(e, w)
    end

    def test_all
      session

      assert(nil, [:a]) # do nothing
      assert(nil, [:a, {:href=>"t"}, "t"])

      assert_equal(false, @site.exist?("t"))
      assert_equal(false, @site.db.baseexist?("t"))
      assert_equal(nil, @site["t"])

      assert([[:span, {:class=>"new"}, "t"],
	       [:a, {:class=>"new", :href=>".new?t=t"},
		 [:img, {:alt=>"create",
		     :src=>"/.theme/i/new.png", :class=>"new"}]]],
	     [:a, {:href=>"t.html"}, "t"])
      page = @site.create("t")
      assert([[:a, {:href=>"t.html"}, "t"]], [:a, {:href=>"t.html"}, "t"])
      page = @site.create_new
      page.store("t")
      assert([[:a, {:href=>"1.html"}, "1"]], [:a, {:href=>"1.html"}, "1"])
      assert([[:a, {:href=>"1.html"}, "t"]], [:a, {:href=>"1.html"}, "t"])
      page.store("*t")
      assert([[:a, {:href=>"1.html"}, "t"]], [:a, {:href=>"1.html"}, "1"])
      assert([[:a, {:href=>"1.html"}, "t"]], [:a, {:href=>"1.html"}, "t"])
      assert([[:a, {:href=>"1.html"}, "s"]], [:a, {:href=>"1.html"}, "s"])

      # test_error
      assert(nil, [:a, {:href=>'"D_R", "/v/w"'}, "t"])

      # test_action
      assert(nil, [:a, {:href=>".attach"}, ".attach"])
    end
  end

  class TC_ResolveEtc < Test::Unit::TestCase
    include TestSession

    def test_all
      # test_br
      assert_wiki([:p, "t", [:br], "s"], "t~\ns\n")
      assert_equal("\342\200\276", "~".sjistou8)

      # test_resolve_ref
      assert_wiki([:p, [:a, {:href=>"FrontPage.html"}, "FrontPage"]],
		  "[[FrontPage]]")
      assert_wiki([:p, [:a, {:href=>"FrontPage.html"}, "t"]], "[[t|FrontPage]]")
      page = @site.create_new
      page.store("*")
      k = page.key
      assert_wiki([:p, [:a, {:href=>"2.html"}, ""]], "[[#{k}]]")
      assert_wiki([:p, [:a, {:href=>"2.html"}, ""]], "[[]]")
      assert_wiki([:p, [:span, {:class=>"new"}, ""],
		    [:a, {:class=>"new", :href=>".new?t=%82%A2"},
		      [:img, {:alt=>"create", :class=>"new",
			  :src=>"/.theme/i/new.png"}]]],
		  "[[]]")

      # test_resolve_plugin
      assert_wiki([:p, [:em, "t"]], "''t''")
      assert_wiki([:p, [:em, ""]], "''{{null}}''")
      assert_wiki([:p, [:strong, "t"]], "'''t'''")
      assert_wiki([:p, [:strong, ""]], "'''{{null}}'''")
      assert_wiki([""], "{{null}}")
      assert_wiki(["nosuch plugin"], "{{nosuchplugin}}")
    end
  end

  class TC_FormatterResolve < Test::Unit::TestCase
    include TestSession

    def assert(e, from)
      s = @action.c_res(from)
      assert_equal(e, s)
    end

    def test_all
      session

      # test_pre
      assert([[:pre, "t"]], " t")
      assert([[:pre, "s\nt"]], " s\n t")

      # test_lines
      assert([[:ul, [:li, "t1"], [:li, "t2"]]], "- t1\n- t2")
      assert([[:p, "t1"], [:ul, [:li, "t2"]]], "t1\n- t2")

      # test_qwik_text
      assert([[:ul, [:li, "t"]]], "- t")
      assert([[:ul, [:li, "s"], [:ul, [:li, "t"]]]], "- s\n-- t")
      assert([[:ol, [:li, "t"]]], "+ t")

      # test_ul
      assert([[:ul, [:li, "t1"]], ["\n"], [:ul, [:li, "t2"]]], "- t1\n\n- t2")
    end
  end

  class TC_Formatter < Test::Unit::TestCase
    include TestSession

    def test_all
      session

      # test_newpage

      assert_wiki([:p, [:span, {:class=>"new"}, "test"],
		    [:a, {:href=>".new?t=test", :class=>"new"},
		      [:img, {:src=>"/.theme/i/new.png",
			  :class=>"new", :alt=>"create"}]]],
		  "[[test]]")

      assert_wiki([:p, [:span, {:class=>"new"}, ""],
		    [:a, {:href=>".new?t=%82%A2", :class=>"new"},
		      [:img, {:src=>"/.theme/i/new.png",
			  :class=>"new", :alt=>"create"}]]],
		  "[[]]")

      # test_plugin_action
      assert_wiki("<p>t<a href=\".attach\">.attach</a></p>", "t[[.attach]]")
      assert_wiki("<p><a href=\".attach\">.attach</a></p>", "[[.attach]]")
      assert_wiki("<p><a href=\".attach\">FileAttach</a></p>",
		  "[[FileAttach|.attach]]")
      assert_wiki("<p><a href=\".attach\">t@CYt</a></p>",
		  "[[t@CYt|.attach]]")

      # test_mojibake
      assert_wiki("<p>t</p>", "t")
      assert_wiki("<p>450~</p>", "450~")

      # test_plugin_link
      assert_wiki("<p><a href=\".recent\">.recent</a></p>", "[[.recent]]")
      assert_wiki("<p><a href=\".recent\">RecentList</a></p>",
		  "[[RecentList|.recent]]")
      assert_wiki("<p><a href=\".rss\">.rss</a></p>",
		  "[[.rss]]") # not use for now
      assert_wiki("<p><a href=\".attach\">.attach</a></p>", "[[.attach]]")
      assert_wiki("<p><a href=\".attach/t.txt\">.attach/t.txt</a></p>",
		  "[[.attach/t.txt]]")
      assert_wiki("<p><a href=\".attach/s t.txt\">.attach/s t.txt</a></p>",
		  "[[.attach/s t.txt]]")

      # test_block
      assert_wiki("<dl><dt>t</dt><dd>s</dd></dl>", ":t:s")
      assert_wiki("<dl><dt>t</dt></dl>", ":t:")
      assert_wiki("<dl><dt>t</dt></dl>", ":t")
      assert_wiki("<blockquote><p>t</p></blockquote>", "> t")
      assert_wiki("<blockquote><p>t</p></blockquote>", ">t")
      assert_wiki("<table><tr><td>t</td><td>s</td></tr></table>", "|t|s|")

      # test_name
      #assert_wiki("<h2><span class='date'/><span class='title'><a name='l0'> </a> t</span></h2>", "* t")

      # test_inline
      assert_wiki("<p><em>t</em></p>", "''t''")
      assert_wiki("<p><strong>t</strong></p>", "'''t'''")

      # test_link
      assert_wiki("<p>t</p>", "t")
      page = @site["FrontPage"]
      page.store("*title")
      assert_wiki("<p><a href=\"FrontPage.html\">title</a></p>", "[[FrontPage]]")
      page = @site.create("t")
      page.store("t2")
      assert_wiki("<p><a href=\"t.html\">t</a></p>", "[[t]]")
      assert_wiki("<p><a href=\"FrontPage.html\">t</a></p>", "[[t|FrontPage]]")
      assert_wiki("<p><a href=\"t.html\">s</a></p>", "[[s|t]]")
      page = @site.create_new
      page.store("*")
      k = page.key
      assert_wiki("<p><a href=\"#{k}.html\"></a></p>", "[[#{k}]]")

      # test_find_title
      assert_wiki("<p><a href=\"#{k}.html\"></a></p>", "[[]]")

      # test_qwik_text
      assert_wiki("<p><del>b</del></p>", "==b==")
      assert_wiki("<hr/>", "====")

      # test_apos
      assert_wiki("<p>'</p>", "'")
      assert_wiki("<p>a</p>", "a")
      assert_wiki("<p>&lt;</p>", "<")

      # test_plugin
      assert_wiki("", "{{null}}")
      assert_wiki("<p>a </p>", "a {{null}}")
      assert_wiki("<p>a  b</p>", "a {{null}} b")
      assert_wiki("<p>a <h2>From: a</h2> b</p>", "a {{mail(a)}} b")
      assert_wiki("<p>a <span class=\"date\">Jan 1, 1970</span> b</p>",
		  "a {{date(0)}} b")
      assert_wiki("<p>a <h2><span class=\"date\">Jan 1, 1970</span> From: a</h2> b</p>", "a {{mail(a,0)}} b")
    end
  end
end
