# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/webrick"
require "qwik/config"
require "qwik/memory"
require "qwik/catalog"
require "qwik/logger"
require "qwik/action"
require "qwik/periodic"
require "qwik/request"
require "qwik/response"
require "qwik/loadlib"

module Qwik
  class Server < WEBrick::HTTPServer
    def initialize(qconfig, memory)
      @qconfig = qconfig
      @memory  = memory

      @qconfig.log_dir.path.check_directory
      @qconfig.etc_dir.path.check_directory
      @qconfig.data_dir.path.check_directory
      (@qconfig.data_dir.path+@qconfig.default_sitename).check_directory
      @qconfig.cache_dir.path.check_directory
      @qconfig.grave_dir.path.check_directory

      @logfile  = @qconfig.log_file.dup		# qwikweb.log
      @alogfile = @qconfig.accesslog_file.dup	# access.log
      @qlogfile = @qconfig.qlog_file.dup	# qwik-access.log
      @pidfile  = @qconfig.pid_file.dup

      loglevel = WEBrick::Log::INFO
      servertype = WEBrick::Daemon
      format = WEBrick::AccessLog::COMBINED_LOG_FORMAT

      @port = @qconfig.port.to_i
      if @qconfig.ssl
	require "webrick/https"
	@logfile  += "-ssl"
	@alogfile += "-ssl"
	@qlogfile += "-ssl"
	@pidfile  += "-ssl"
	@port = 443 if @port == 80
      end

      if @qconfig.debug
	@pidfile += "-d"
	#loglevel = WEBrick::Log::DEBUG
	servertype = WEBrick::SimpleServer
	#format = "%l %u \"%r\" %s %b"
	if @qconfig.test
	  @logfile  = "testlog.txt"
	else
	  @logfile  = $stdout
#	  @alogfile = $stdout
#	  @qlogfile = $stdout
	end
      end

      logger = WEBrick::Log.new(@logfile, loglevel)
      @memory[:logger] = logger

      qlog = Qwik::AccessLog.new(@qconfig, @qlogfile)
      @memory[:accesslog] = qlog

      basic_log = WEBrick::BasicLog.new(@alogfile, loglevel)
     #server_software = "qwikWeb/#{@qconfig.version}+#{@qconfig.release_date}"
     #server_software = "qwikWeb/"+@qconfig.version+"+"+@qconfig.release_date
      server_software = "qwikWeb/"+Config::VERSION+"+"+Config::RELEASE_DATE

      webrick_config = {
	:HostnameLookups => false,
	:BindAddress    => @qconfig.bind_address,
	:Port           => @port,
	:Logger         => logger,
	:ServerType     => servertype,
	:StartCallback  => Proc.new { start_server },
	:AccessLog      => [[ basic_log, format ]],
	:ServerSoftware => server_software,
	:QwikConfig     => @qconfig,
	:ServerMemory   => @memory,
	:StopCallback   => Proc.new { p "stop"; }, # for test
#	:WEBrickThread  => false, # test
      }

      if @qconfig.ssl
	sslconfig = {
	  :SSLEnable       => true,
	  :SSLVerifyClient => ::OpenSSL::SSL::VERIFY_NONE,
	  :SSLCertName => [ ["C","JP"], ["O","example.com"], ["CN", "WWW"] ]
	}
	webrick_config.update(sslconfig)
      end

      super(webrick_config)

      loadlib = LoadLibrary.new(@qconfig.lib_dir)
      loadlib.glob("qwik/act-*.rb")

      mount("/", Servlet)
    end

    def start_server
      write_pid_file

      @threads = []
      @threads <<            SweepThread.new(@qconfig, @memory)
      @threads << WeeklySendReportThread.new(@qconfig, @memory)
      @threads <<  DailySendReportThread.new(@qconfig, @memory)
      @threads << HourlySendReportThread.new(@qconfig, @memory)
      @memory[:threads] = @threads
      @threads.each {|th|
	t = Thread.new { th.start }
	t.abort_on_exception = true
      }

      if @qconfig.debug
	require "qwik/autoreload"
	autoreload(1, true) # auto reload
      end
    end

    def write_pid_file
      File.open(@pidfile, "wb") {|f| f.puts Process.pid }
    end

    def read_pid_file
      open(@pidfile, "rb"){|f| f.read}.chomp.to_i
    end

    def remove_pid_file
      File.unlink(@pidfile) if Process.pid == read_pid_file
    end
  end

  class Servlet < WEBrick::HTTPServlet::AbstractServlet
    def do_GET(request, response)
      config = @server[:QwikConfig]
      memory = @server[:ServerMemory]
      req = Request.new(config)
      req.parse_webrick(request)
#      pp req
#      pp req.header
#      pp request

      res = Response.new(config)
      res.set_webrick(response)

      action = Action.new(config, memory, req, res)
      action.run

      res.setback(response)

      qlog = memory[:accesslog]
      qlog.log(request, response, req, res) if qlog # take log

      if res.basicauth
	proc = res.basicauth
	WEBrick::HTTPAuth::basic_auth(request, response, "qwik", &proc)
      end
    end
    alias do_POST do_GET

  end
end
