# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

module Qwik
  class Action
    def c_page_url
      page = @site[@req.base]
      page.url
    end
  end

  class TDiaryResolver
    def initialize(config, site, action)
      @config = config
      @site = site
      @action = action

      @global_id = 0

      t = @site.siteconfig["titlelink"]
      @titlelink = (t == "true")

      t = @site.siteconfig["emode_titlelink"]
      @emode_titlelink = (t == "true")
    end
    attr_reader :titlelink, :emode_titlelink # for test

    def resolve(wabisabi)
      days = []
      section = []
      days << section
      wabisabi.each {|e|
	if is_title?(e)
	  if 0 < section.size
	    section = []
	    days << section
	  end
	end
	section << e
      }

      @global_id = 0
      ar = []
      days.each {|section|
	title = ""
	title = section.shift if is_title?(section[0])
	ar += tdiary_section(title, section)
      }
      ar
    end

#    private	# for test

    def is_title?(e)
      e.is_a?(Array) && e[0] == :h2
    end

    def encode_label(name)
      if name == ""
	@global_id += 1
	return @global_id.to_s
      end

      if /\A[A-Za-z0-9\-_:. ]+\z/ =~ name
	name.gsub!(/[^A-Za-z0-9\-_]+/, '_')
	name.squeeze!('_')
	name.chomp!('_')
	name.sub!(/\A_/, '')
	name = 'a' + name if /\A[A-Za-z]/ !~ name
	return name
      end

      if @emode_titlelink
	name = name+'<span class="e"><br></span>' # adhoc
	name = name.md5.base64
	name.gsub!(/=/, "")
	name.gsub!(/\+/, "-")
	name.gsub!(/\//, "_")
	return name
      end

      return name.md5hex
    end

    def get_title_name(e)
      (1..e.length).each {|i|
	if e[i].is_a?(String)
	  return e[i].to_s.strip
	end
      }
      return ""
    end

    def make_title_link(e)
      return e if e.length < 2

      name = get_title_name(e)
      label = encode_label(name)

      url = @action.c_page_url
      href = "#{url}#"+label

      if (@titlelink || @emode_titlelink)
	m = ""
	e.insert(1, [:a, {:class=>"label", :href=>href, :name=>label}, m])
	return e 
      end

      e.insert(1, {:id=>label})
      e
    end

    def tdiary_section(title, section)
      title = make_title_link(title)
      return [
	[:div, {:class=>"day"},
	  title,
	  [:div, {:class=>"body"},
	    [:div, {:class=>"section"},
	      section],
	    [:"!--", "section"]],
	  [:"!--", "body"]],
	[:"!--", "day"]
      ]
    end
  end
end

if $0 == __FILE__
  $LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
  require "t/test"
  $debug = true
end

if $debug
  class TC_TDiaryResolver < Test::Unit::TestCase
    include TestSession

    def assert_t(e, s)
      res = Qwik::TDiaryResolver.new(@config, @site, @action)
      assert_equal(e, res.make_title_link(s))
    end

    def test_basic
      session

      res = Qwik::TDiaryResolver.new(@config, @site, @action)

      # test_is_title?
      assert_equal(true,  res.is_title?([:h2, "t"]))
      assert_equal(false, res.is_title?([:h3, "t"]))

      # test_encode_label
      assert_equal("1",  res.encode_label(""))
      assert_equal("2",  res.encode_label(""))
      assert_equal("t",  res.encode_label("t"))
      assert_equal("3",  res.encode_label(""))

      assert_equal("t_t",  res.encode_label("t t"))
      assert_equal("t_t",  res.encode_label("t_t"))
      assert_equal("t-t",  res.encode_label("t-t"))
      assert_equal("930149ca7573114b0341159c94380421", res.encode_label("t!"))

      # test_tdiary_section
      t = res.tdiary_section([:h2, "t"], "section")
      assert_equal([[:div, {:class=>"day"},
		       [:h2, {:id=>"t"}, "t"],
		       [:div, {:class=>"body"},
			 [:div, {:class=>"section"}, "section"],
			 [:"!--", "section"]],
		       [:"!--", "body"]],
		     [:"!--", "day"]],
		   t)

      assert_equal([:div, {:class=>"day"},
		     [:h2, {:id=>"t"}, "t"],
		     [:div, {:class=>"body"},
		       [:div, {:class=>"section"}, "section"]]],
		   t.remove_comment.get_single)

      # test_make_title_link
      session

      assert_t([:h2, {:id=>"t"}, "t"], [:h2, "t"])

      config = @site["_SiteConfig"]
      config.store(":titlelink:true")

      assert_t([:h2, [:a, {:href=>"FrontPage.html#t", :name=>"t",
		     :class=>"label"}, ""], "t"],
	       [:h2, "t"])

      assert_t([:h2, [:a, {:name=>"8f03c3a6dbec1d0f1a5af60947b7b052",
		     :class=>"label",
		     :href=>"FrontPage.html#8f03c3a6dbec1d0f1a5af60947b7b052"},
		   ""], ""],
	       [:h2, ""])
    end

    def test_emode
      session

      config = @site["_SiteConfig"]
      config.store(":emode_titlelink:true")
      res = Qwik::TDiaryResolver.new(@config, @site, @action)
      assert_equal("t",  res.encode_label("t"))
      assert_equal("zNC0vEuG7ZsGqzX0C5tyRQ",  res.encode_label("t!"))
      assert_equal("ZHQyWazgdpeYgxXBvfV-jA",  res.encode_label(""))

      assert_t([:h2, [:a, {:href=>"FrontPage.html#t", :name=>"t",
		     :class=>"label"}, ""], "t"],
	       [:h2, "t"])
      assert_t([:h2, [:a, {:name=>"ZHQyWazgdpeYgxXBvfV-jA",
		     :class=>"label",
		     :href=>"FrontPage.html#ZHQyWazgdpeYgxXBvfV-jA"},
		   ""], ""],
	       [:h2, ""])
    end

    def assert(e, wabisabi)
      res = Qwik::TDiaryResolver.new(@config, @site, @action)
      w = res.resolve(wabisabi).remove_comment
      assert_equal(e, w)
    end

    def test_tdiary_resolve
      session
      assert([[:div, {:class=>"day"}, "",
		 [:div, {:class=>"body"}, [:div, {:class=>"section"}, []]]]],
	     "")
      assert([[:div, {:class=>"day"},
		 [:h2, {:id=>"t"}, "t"],
		 [:div, {:class=>"body"}, [:div, {:class=>"section"}, []]]]],
	     [[:h2, "t"]])
      assert([[:div, {:class=>"day"},
		 [:h2, {:id=>"h2"},"h2"],
		 [:div, {:class=>"body"},
		   [:div, {:class=>"section"},
		     [[:p, "text of h2"], [:h3, "h3"], [:p, "text of h3"]]]]]],
	     [[:h2, "h2"],
	       [:p, "text of h2"],
	       [:h3, "h3"],
	       [:p, "text of h3"]])
      assert([[:div, {:class=>"day"},
		 [:h2, {:id=>"h2"},"h2"],
		 [:div, {:class=>"body"},
		   [:div, {:class=>"section"}, [[:p, "text of h2"]]]]],
	       [:div, {:class=>"day"},
		 [:h2, {:id=>"h2_2"},"h2 2"],
		 [:div, {:class=>"body"},
		   [:div, {:class=>"section"}, [[:p, "text of h2 2"]]]]]],
	     [[:h2, "h2"],
	       [:p, "text of h2"],
	       [:h2, "h2 2"],
	       [:p, "text of h2 2"]])
    
      # test_tdiary_resolve_with_titlelink
      config = @site["_SiteConfig"]
      config.store(":titlelink:true")
      assert([[:div, {:class=>"day"},
		 [:h2, [:a, {:name=>"t", :class=>"label",
		       :href=>"FrontPage.html#t"}, ""], "t"],
		 [:div, {:class=>"body"}, [:div, {:class=>"section"}, []]]]],
	     [[:h2, "t"]])
    end
  end
end
