# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

class String
  def escape # copied from cgi.rb
    self.gsub(/([^ a-zA-Z0-9_.-]+)/n) {
      "%" + $1.unpack("H2" * $1.size).join("%").upcase
    }.tr(" ", "+")
  end

  def unescape
    self.tr("+", " ").gsub(/((?:%[0-9a-fA-F]{2})+)/n) {
      [$1.delete("%")].pack("H*")
    }
  end

  def unescapeHTML
    self.gsub(/&(.*?);/n) {
      match = $1.dup
      case match
      when /\Aamp\z/ni	then "&"
      when /\Aquot\z/ni	then "'"
      when /\Agt\z/ni	then ">"
      when /\Alt\z/ni	then "<"
      else
	"&#{match};"
      end
    }
  end

  # copy from gonzui-0.1
  # Use this code instead of WEBRick::HTMLUtils.escape for performance reason.
  EscapeTable = {
    "&" => "&amp;",
    '"' => "&quot;",
    '<' => "&lt;",
    '>' => "&gt;",
  }
  def escapeHTML
    string = self
    string.gsub(/[&"<>]/n) {|x| EscapeTable[x] }
  end
end

$debug = true if $0 == __FILE__
if $debug
  require "test/unit"

  class TC_Escape < Test::Unit::TestCase
    def test_escape
      assert_equal("A", "A".escape)
      assert_equal("+", " ".escape)
      assert_equal("%2B", "+".escape)
      assert_equal("%21", "!".escape)
      assert_equal("A", "%41".unescape)
      assert_equal(" ", "+".unescape)
      assert_equal("!", "%21".unescape)
      assert_equal("&lt;", "<".escapeHTML)
      assert_equal("<", "&lt;".unescapeHTML)
      assert_equal("&gt;", ">".escapeHTML)
      assert_equal(">", "&gt;".unescapeHTML)
      assert_equal("&amp;", "&".escapeHTML)
      assert_equal("&", "&amp;".unescapeHTML)
      assert_equal("ABC%82%A0%82%A2%82%A4+%2B%23", "ABC +#".escape)
      assert_equal("ABC +#", "ABC%82%A0%82%A2%82%A4+%2B%23".unescape)
      assert_equal("&lt;a href=&quot;http://e.com/&quot;&gt;e.com&lt;/a&gt;",
		   '<a href="http://e.com/">e.com</a>'.escapeHTML)
      assert_equal("<a href='http://e.com/'>e.com</a>",
		   "&lt;a href=&quot;http://e.com/&quot;&gt;e.com&lt;/a&gt;".unescapeHTML)
    end
  end
end
