# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

require "iconv"

class Iconv
  def self.iconv_to_utf8(from, str)
    iconv = Iconv.new(from, "UTF-8")
    out = ""
    begin
      out << iconv.iconv(str)
    rescue Iconv::IllegalSequence => e
      out << e.success
      ch, str = e.failed.split(//u, 2)
      out << if respond_to?(:unknown_unicode_handler)
	       u = ch.unpack("U").first
	       unknown_unicode_handler(u)
	     else
	       "?"
	     end
      retry
    end
    out
  end

  def self.unknown_unicode_handler (u)
    sprintf("&#x%04x;", u)
  end

  def self.iconv_to_from(to, from, str)
    iconv = Iconv.new(to, from)
    out = ""
    begin
      out << iconv.iconv(str)
    rescue Iconv::IllegalSequence => e
      out << e.success
      ch, str = e.failed.split(//u, 2)
      out << "?"
      retry
    rescue Iconv::InvalidCharacter => e
      out << e.success
      ch, str = e.failed.split(//u, 2)
      out << "?"
      retry
    end
    out
  end
end

class String
  def euctou8()  Iconv.iconv_to_from("UTF-8", "EUC-JP", self)		end
  def u8toeuc()  Iconv.iconv_to_from("EUC-JP", "UTF-8", self)		end
  def sjistou8() Iconv.iconv_to_from("UTF-8", "Shift_JIS", self)	end
  def u8tosjis() Iconv.iconv_to_from("Shift_JIS", "UTF-8", self)	end
  def jistou8()  Iconv.iconv_to_from("UTF-8", "ISO-2022-JP", self)	end
  def u8tojis
    i = Iconv.new("ISO-2022-JP", "UTF-8")
    i.iconv(self)+i.close
  end
end

$debug = true if $0 == __FILE__
if $debug
  $LOAD_PATH.unshift("../../compat") unless $LOAD_PATH.include?("../../compat")
  $LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
  require "test/unit"
  require "qwik/util-kconv"

  class TC_Iconv < Test::Unit::TestCase
    def test_iconv
      assert_equal("", "".sjistou8.u8tosjis)
      assert_equal("", "".sjistoeuc.euctosjis)
      assert_equal("", "".sjistojis.jistosjis)
      assert_equal("", "\264\301\273\372".euctosjis)
      assert_equal("\e$B4A;z\e(B", "".sjistojis)
    end
  end
end
