#!/usr/bin/env ruby
# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.

require "test/unit"
$LOAD_PATH.unshift("../../compat") unless $LOAD_PATH.include?("../../compat")
require "qrcode"
$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "qwik/config"
require "qwik/qp"
require "qwik/qrcode-image"
require "qwik/qrcode-html"
require "qwik/format-xml"

# $KCODE = "s" # qrcode library should be free from $KCODE

class TC_QRCodeHtml < Test::Unit::TestCase
  def setup
    config = Qwik::Config.new(true)
    path = config.lib_dir+"/qrcode"
    @q = QRCodeHtml.new(path)
  end

  def test_qrcode_htree
    qrcode = @q.instance_eval { @qrcode }
    d = nil
    (1..1).each { # 100times: 23.628 seconds.
      d = qrcode.make_qrcode("0")
      assert_equal("111111101111101111111\n100000100110101000001\n101110100010001011101\n101110101110001011101\n101110101010101011101\n100000101111001000001\n111111101010101111111\n000000001001100000000\n100010111111011111001\n001010010011100101011\n100100101011001111100\n111010001100011010100\n100000111100111000111\n000000001000111000111\n111111101100110000010\n100000100001100101000\n101110101011001111111\n101110100011100101011\n101110100101001111100\n100000100100011010110\n111111101100111000111\n", d)
    }

    h = nil
    (1..1).each {
      h = @q.generate_htree_from_data(d)
    }
    assert_equal("<table class=\"qrcode\"><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><th/><th/><th/><th/><td/><th/><th/><th/><th/><th/><th/><th/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><td/><th/><th/><td/><th/><td/><th/><td/><td/><td/><td/><td/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><td/><td/><th/><td/><td/><td/><th/><td/><th/><th/><th/><td/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><th/><th/><th/><td/><td/><td/><th/><td/><th/><th/><th/><td/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><th/><td/><th/><td/><th/><td/><th/><td/><th/><th/><th/><td/><th/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><th/><th/><th/><th/><td/><td/><th/><td/><td/><td/><td/><td/><th/></tr><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><td/><th/><td/><th/><td/><th/><th/><th/><th/><th/><th/><th/></tr><tr><td/><td/><td/><td/><td/><td/><td/><td/><th/><td/><td/><th/><th/><td/><td/><td/><td/><td/><td/><td/><td/></tr><tr><th/><td/><td/><td/><th/><td/><th/><th/><th/><th/><th/><th/><td/><th/><th/><th/><th/><th/><td/><td/><th/></tr><tr><td/><td/><th/><td/><th/><td/><td/><th/><td/><td/><th/><th/><th/><td/><td/><th/><td/><th/><td/><th/><th/></tr><tr><th/><td/><td/><th/><td/><td/><th/><td/><th/><td/><th/><th/><td/><td/><th/><th/><th/><th/><th/><td/><td/></tr><tr><th/><th/><th/><td/><th/><td/><td/><td/><th/><th/><td/><td/><td/><th/><th/><td/><th/><td/><th/><td/><td/></tr><tr><th/><td/><td/><td/><td/><td/><th/><th/><th/><th/><td/><td/><th/><th/><th/><td/><td/><td/><th/><th/><th/></tr><tr><td/><td/><td/><td/><td/><td/><td/><td/><th/><td/><td/><td/><th/><th/><th/><td/><td/><td/><th/><th/><th/></tr><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><th/><td/><td/><th/><th/><td/><td/><td/><td/><td/><th/><td/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><td/><td/><td/><th/><th/><td/><td/><th/><td/><th/><td/><td/><td/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><th/><td/><th/><th/><td/><td/><th/><th/><th/><th/><th/><th/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><td/><td/><th/><th/><th/><td/><td/><th/><td/><th/><td/><th/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><td/><th/><td/><th/><td/><td/><th/><th/><th/><th/><th/><td/><td/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><td/><th/><td/><td/><td/><th/><th/><td/><th/><td/><th/><th/><td/></tr><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><th/><td/><td/><th/><th/><th/><td/><td/><td/><th/><th/><th/></tr></table>",
		 h.rb_format_xml(-1, -1))
  end

  def test_qrcode_html
    qrcode = @q.instance_eval { @qrcode }
    d = qrcode.make_qrcode("0")

    h = nil
    (1..1000).each {
      h = @q.generate_html_from_data(d)
    }
    assert_equal("<table class=\"qrcode\"><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><th/><th/><th/><th/><td/><th/><th/><th/><th/><th/><th/><th/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><td/><th/><th/><td/><th/><td/><th/><td/><td/><td/><td/><td/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><td/><td/><th/><td/><td/><td/><th/><td/><th/><th/><th/><td/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><th/><th/><th/><td/><td/><td/><th/><td/><th/><th/><th/><td/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><th/><td/><th/><td/><th/><td/><th/><td/><th/><th/><th/><td/><th/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><th/><th/><th/><th/><td/><td/><th/><td/><td/><td/><td/><td/><th/></tr><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><td/><th/><td/><th/><td/><th/><th/><th/><th/><th/><th/><th/></tr><tr><td/><td/><td/><td/><td/><td/><td/><td/><th/><td/><td/><th/><th/><td/><td/><td/><td/><td/><td/><td/><td/></tr><tr><th/><td/><td/><td/><th/><td/><th/><th/><th/><th/><th/><th/><td/><th/><th/><th/><th/><th/><td/><td/><th/></tr><tr><td/><td/><th/><td/><th/><td/><td/><th/><td/><td/><th/><th/><th/><td/><td/><th/><td/><th/><td/><th/><th/></tr><tr><th/><td/><td/><th/><td/><td/><th/><td/><th/><td/><th/><th/><td/><td/><th/><th/><th/><th/><th/><td/><td/></tr><tr><th/><th/><th/><td/><th/><td/><td/><td/><th/><th/><td/><td/><td/><th/><th/><td/><th/><td/><th/><td/><td/></tr><tr><th/><td/><td/><td/><td/><td/><th/><th/><th/><th/><td/><td/><th/><th/><th/><td/><td/><td/><th/><th/><th/></tr><tr><td/><td/><td/><td/><td/><td/><td/><td/><th/><td/><td/><td/><th/><th/><th/><td/><td/><td/><th/><th/><th/></tr><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><th/><td/><td/><th/><th/><td/><td/><td/><td/><td/><th/><td/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><td/><td/><td/><th/><th/><td/><td/><th/><td/><th/><td/><td/><td/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><th/><td/><th/><th/><td/><td/><th/><th/><th/><th/><th/><th/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><td/><td/><th/><th/><th/><td/><td/><th/><td/><th/><td/><th/><th/></tr><tr><th/><td/><th/><th/><th/><td/><th/><td/><td/><th/><td/><th/><td/><td/><th/><th/><th/><th/><th/><td/><td/></tr><tr><th/><td/><td/><td/><td/><td/><th/><td/><td/><th/><td/><td/><td/><th/><th/><td/><th/><td/><th/><th/><td/></tr><tr><th/><th/><th/><th/><th/><th/><th/><td/><th/><th/><td/><td/><th/><th/><th/><td/><td/><td/><th/><th/><th/></tr></table>", h)
  end
end

class TC_QRCode < Test::Unit::TestCase
  def setup
    config = Qwik::Config.new(true)
    path = config.lib_dir+"/qrcode"
    @q = QRCode.new(path)
  end

  def assert_qrcode(expected, data)
    assert_equal(expected, @q.qrcode_ar(data))
  end

  def test_qrcode
    assert_equal("111111101111101111111\n100000100110101000001\n101110100010001011101\n101110101110001011101\n101110101010101011101\n100000101111001000001\n111111101010101111111\n000000001001100000000\n100010111111011111001\n001010010011100101011\n100100101011001111100\n111010001100011010100\n100000111100111000111\n000000001000111000111\n111111101100110000010\n100000100001100101000\n101110101011001111111\n101110100011100101011\n101110100101001111100\n100000100100011010110\n111111101100111000111\n", @q.make_qrcode("0"))
    assert_equal(0, @q.qrcode_version)
    assert_equal(1, @q.qrcode_version_used)
    assert_qrcode("1fdf7f:104d41:17445d:175c5d:17555d:105e41:1fd57f:1300:117ef9:5272b:12567c:1d18d4:1079c7:11c7:1fd982:104328:17567f:17472b:174a7c:1048d6:1fd9c7", "0")
    assert_qrcode("1fdb7f:105441:174d5d:17585d:174c5d:104c41:1fd57f:1800:16e44b:2b52c:12f1f2:1a258a:3f290:1ca1:1fd0d6:1057c5:174a41:175496:175694:104b5b:1fd322", "01234567")
    assert_qrcode("1fdf7f:105c41:174f5d:17555d:174c5d:104041:1fd57f:1c00:16e44b:e252c:10cdf2:c2d8a:1dea90:18a1:1fd4d4:1057c1:17464d:17549a:175a90:104b5b:1fd722", "012345678")
    assert_qrcode("1fd837f:1053141:175b35d:174415d:175a75d:104f341:1fd557f:5b00:13f0997:b9f93e:64aba9:1e3da6f:1561941:b98e12:3ebb0f:b26195:cec7f6:13f12:1fda159:1055712:175dffa:1750469:17457b7:104c437:1fd8d89", "http://example.com/")
    assert_qrcode("1fce17f:1046a41:175655d:175cc5d:175885d:1054f41:1fd557f:1c500:17c887c:4ba9a2:b51ffb:4261a1:3d9c77:b092a:fd3abb:ca2229:d6adf4:1d71c:1fcc15f:105911a:1755bff:175c8f7:1753ea5:104a739:1fdb0ff", "http://www.yahoo.co.jp/")
  end

  def nutest_qrcode2
    assert_qrcode("1fd77f:104e41:17475d:17525d:17515d:105641:1fd57f:1700:117ef9:18af2f:11d273:618d0:bedc3:1dcf:1fd18d:10472d:175a73:174f0f:17467c:104cfe:1fddcf", "0123456789")
    assert_qrcode("1fca7f:104b41:17495d:17445d:17415d:105341:1fd57f:700:12d0a0:19a842:e4d8c:8e0b:dc151:1e30:1fc0ab:1057bf:174673:17562b:174111:1048fe:1fd330", "01234567890")
    assert_qrcode("1fdd7f:105541:17475d:175a5d:17425d:104341:1fd57f:1b00:16eb4b:141fc8:13560d:13967c:17cd24:1a48:1fd328:105836:1745f1:175e7e:175d60:1044a5:1fd890", "a")
    assert_qrcode("1fd27f:104641:17495d:175e5d:17535d:105941:1fd57f:1800:1175f9:1db9cc:8498a:e1320:dda7a:1f2e:1fd27a:1040d0:175dc9:1749c7:174d88:104328:1fda75", "http")
    assert_qrcode("1fc67f:105541:17565d:175d5d:17415d:104e41:1fd57f:1800:105ece:1fb56e:7186:1a10bc:12ee02:109d:1fcfe6:1048ef:17465b:174454:17477f:104294:1fd87a", "http://qwik.j")
    assert_qrcode("1fc47f:105041:175a5d:17555d:17475d:104d41:1fd57f:1f00:1055ce:1c195d:ae69f:1db41c:ee84b:13cc:1fc962:104392:174130:174d0c:17485b:10442a:1fd2a0", "0123456789012")
    assert_qrcode("1fce7f:105341:17535d:175c5d:17415d:104e41:1fd57f:1400:105ace:d17bd:1a555f:18bcc:1f4beb:1d5c:1fce30:104c43:174a12:174fcc:17439b:104bea:1fdd20", "01234567890123456789")
    assert_qrcode("1fdf7f:104341:174d5d:175d5d:17555d:105e41:1fd57f:1b00:117af9:178d2f:187ac3:1082b0:15e713:139f:1fd06d:104a2f:175271:174d0f:174b5c:104ed6:1fda43", "012345678901234567890123456789")
    assert_qrcode("1fdb7f:105441:174d5d:17585d:174c5d:104c41:1fd57f:1800:16e44b:2b52c:12f1f2:1a258a:3f290:1ca1:1fd0d6:1057c5:174a41:175496:175694:104b5b:1fd322", "01234567")
    assert_qrcode("1fdf7f:105c41:174f5d:17555d:174c5d:104041:1fd57f:1c00:16e44b:e252c:10cdf2:c2d8a:1dea90:18a1:1fd4d4:1057c1:17464d:17549a:175a90:104b5b:1fd722", "012345678")
    assert_qrcode("1fc2527f:104ead41:175e745d:175ded5d:1758125d:10512c41:1fd5557f:185000:17cf2d7c:1a939251:1447f1be:198684d3:7697fae:18381259:8712092:172b7050:56d2fae:baed259:fc1f93a:cb8a782:d5a6dfe:151319:1fcc2b56:105f5113:17512dfc:175159ab:175d765e:104f6712:1fd4dc0c", "012345678901234567890123456789fadsfdsa")
#    assert_qrcode("1fdf7f:104341:174d5d:175d5d:17555d:105e41:1fd57f:1b00:117af9:178d2f:187ac3:1082b0:15e713:139f:1fd06d:104a2f:175271:174d0f:174b5c:104ed6:1fda43", "abcdefghijklmnopqrstuvwxyz")
    assert_qrcode("1fda77f:104ad41:175175d:174665d:174fc5d:1051041:1fd557f:3600:1469d25:fb0f89:1949329:1211b56:445a09:15311a8:2d9f08:1d1753c:4fcdfb:10d10:1fd2357:1048919:1743bfe:174106c:1759dcb:10436c6:1fd8deb", "0123456789012345678901234567890123456789")
    assert_qrcode("1fd43af9f29c67f:10561da0e61fa41:174edf10850365d:1756be00070e25d:174a9347fbc1a5d:104f6a7461ff441:1fd55555555557f:1786ac4f5ca00:16e2c65fc27c94b:1aa63b32c5bf3b1:34f99c9f32c527:f228b037ae7d08:dd0a2562502842:1af2d1a28c08c0:276765d70f43f0:3a098addafdc5c:176472f6f86f72d:14039da69c3ae91:145815f1c6164a:53e803d491c270:16dca85746f0f57:596efb2b01e3a1:1e91d3d974d40b:41ab2d2a5c3489:18e63a24730a288:168e2b4809c28c4:ff61b1fd1c61fc:1b1fca345871d1c:135db3f5786f95e:1b1cd824771af15:ffbbe7fd15bdf6:60f2fce4a90190:15d4716043b4dc4:131101000bc308:11c46489251c6da:538708b71cb7ca:1cdf1d426b46570:10125e032bf0a28:eea4e48a2d4328:f0f0e75bcdde7f:1c5bb63df02f69c:10b46bef0e79f79:10f59ccecd20c16:d09eccfffde503:65e2b5026fc52e:99cbf4e08f205:14c12eeb940e4c3:1f3bfcd1bb833f8:51ba67ef8a5f8:19790c6cd1110:1fd8c1c574c4354:105570745f7111c:174987aff2ab5fc:1756523cbe2af86:17583e00ed727c8:104eb75c2b34ac1:1fd161d0467c7f4", "http://qwik/shfdosadhfashofhasodufhosduahfuoasdsdfsdafsadhfkjdshafkjahdskfjhdsakhfsalkhfkdashkjsdfhaskjlfhlkajsdhflkjashdlkfjhasdlkjfhlskajdhfkjsahdljkfhsalkjdfhlksjadhfkjsadhfjklsahdlkjfhaskj")
  end
end

#class TC_QRCodeImage < Test::Unit::TestCase
class TC_QRCodeImage
  def setup
    config = Qwik::Config.new(true)
    path = config.lib_dir+"/qrcode"
    @q = QRCodeImage.new(path)
  end

  def test_mkimage
    assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000$\000\000\000$\001\003\000\000\000\333\316\n\333\000\000\000\003PLTE\377\377\377\247\304\e\310\000\000\000\fIDAT\010\231c`\030\236\000\000\000\330\000\001lm\022\026\000\000\000\000IEND\256B`\202", @q.mkimage("0").pngStr)
    assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000$\000\000\000$\001\003\000\000\000\333\316\n\333\000\000\000\006PLTE\377\377\377\000\000\000U\302\323~\000\000\000\021IDAT\010\231c`\2409\370\200\203\244)\000\000\225\347\003\301\252\215\336\360\000\000\000\000IEND\256B`\202", @q.mkimage("1").pngStr)
    assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000\003\000\000\000\003\001\003\000\000\000l\346'\374\000\000\000\006PLTE\377\377\377\000\000\000U\302\323~\000\000\000\016IDAT\010\231c``p``\000\000\000\306\000A\316q\f\035\000\000\000\000IEND\256B`\202", @q.mkimage("1", 1, 1).pngStr)
    assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000t\000\000\000t\001\003\000\000\000nT\204d\000\000\000\006PLTE\377\377\377\000\000\000U\302\323~\000\000\000\227IDAT\010\231\345\223A\016\3040\010\003\375\003\376\377K\377\300\v\244\332l\366T\334cQ\3250\034\"0\016\360\372\220\304\376D\213\363\317 \331\231\303\312\234\353pY\017\271\302\346*\t\307|\023.!\353\272_}'\334w\306\316\247\\RH\261Z2\030d@\214\010\217\223r\032|\307\231r\355AX\216r\230%iV\261\375<\342U\363\271\337Sv\204\200\305m\244H=ar\331I\330\357\331\340c\237\006g3{\277SnKD\365d\361\245\347\377>n\363\253\343\003mkMW\213\010\260\336\000\000\000\000IEND\256B`\202", @q.generate("0"))
    assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000t\000\000\000t\001\003\000\000\000nT\204d\000\000\000\006PLTE\377\377\377\000\000\000U\302\323~\000\000\000\232IDAT\010\231\345\223\273\r\3040\fC\265\001\367\337\222\e\350H\0050\214t\342\2251\202\310\217\205\240\017]\365\371\323\335l\320!b\375IN\310\270IHQ\210\271\237\0203\370\a\353\303\253\277\r{\220n\347\236\357\206G{\357t\301Zf\253\024\002\214\270\000\257\343\340\226uS'\352\006\0317\n*Ji3.8\027\352Ls\313\316i\341R6<\373,\214)\023\236\367d3\\~\\\261\375\214\327{\\\262\224:\365$\254\245\220!\217%,1\342\231'hWD\374\351\363\003\026)\024\333\322(\207<\000\000\000\000IEND\256B`\202", @q.generate("01234567"))
  end
end
