#!/usr/bin/env ruby
# Copyright (C) 2003-2005 Kouichirou Eto, All rights reserved.
# This file is in Shift-JIS character encoding.  Japanese Character -> ""

# $VERBOSE = true
$LOAD_PATH.unshift("../../compat") unless $LOAD_PATH.include?("../../compat")
$LOAD_PATH << "../../lib" unless $LOAD_PATH.include?("../../lib")
require "test/unit"
require "pathname"
require "quickml/quickml"
require "qwik/quickml"
require "t/module-session"
require "pp"

$quickml_debug = true

module QuickML
  class QuickML
    def test(t)
      # test private method
      t.assert_equal("test@e...", obfuscate_address("test@example.com"))
      #t.assert_equal("http://example.com/test/", qwik_url)
      t.assert_equal("http://qwik.jp/test/", qwik_url)
      #t.assert_equal("\n-- \narchive-> http://example.com/test/ \nML-> test@example.com\n", generate_footer)
      t.assert_equal("\n-- \narchive-> http://qwik.jp/test/ \nML-> test@example.com\n", generate_footer)
      t.assert_instance_of(Time, last_article_time)

      # test member file
      t.assert_equal("./test/,members", @members_file)
      t.assert_equal("./test/,count", @count_file)
      t.assert_equal(false, File.exist?(@members_file))

      save_member_file
      t.assert_equal(true, File.exist?(@members_file))
      t.assert_equal("", members_file_read)

      add_member("user@e.com")
      t.assert_equal("user@e.com\n", members_file_read)

      add_error_member("user@e.com")
      t.assert_match(/user@e.com\n; user@e.com 1 /, members_file_read)

      remove_member("user@e.com")
      t.assert_equal("# user@e.com\n", members_file_read)
    end

    def members_file_read
      open(@members_file){|f| f.read }
    end

    def test_submit(t)
      # setup config
      if !defined?($test_config)
	config = Qwik::Config.new(true)
	config.test = true	# do not send mail
	config.pass_file = "./password.txt"
	config.data_dir = "."
	config.cache_dir = "."
	config.db = "bdb"
	$test_org_data_dir = config.data_dir.dup
	$test_config = config
      end
      @qwikconfig = $test_config
      @org_config_db = @qwikconfig.db
      @qwikconfig.db = "fsdb"

      # setup memory
      if !defined?($test_memory)
	memory = Qwik::ServerMemory.new(@qwikconfig)
	$test_memory = memory
      end
      @qwikmemory = $test_memory

      @site = @qwikmemory.farm.get_site("test")
      @site.erase_all
      t.assert_equal(nil, @site["1"])
#     t.assert_equal("eXg", @site["1"].get_title)
#     t.assert_equal("* eXg\n{{mail(user@e.com,0)\n̓eXgłB\n}}\n", @site["1"].load)

      mail = Mail.new
      mail.mail_from = "user@e.com"
      mail.add_recipient("test@example.com")
      str = <<"EOT".sjistojis
From: "Test User" <user@e.com>
To: "Test Mailing List" <test@example.com>
Subject: Re: [test:1] eXg 
Date: Mon, 3 Feb 2001 12:34:56 +0900

̓eXgłB
EOT
      mail.read(str)

      # test_mail
      t.assert_equal(["test@example.com"], mail.recipients)
      t.assert_equal(nil, mail.charset)
      t.assert_equal(nil, mail.content_type)
      t.assert_equal("user@e.com", mail.mail_from)
      t.assert_equal("̓eXgłB\n".sjistojis, mail.body)
      t.assert_equal(str, mail.bare)
      t.assert_equal("Re: [test:1] eXg ".sjistojis, mail["Subject"])
      t.assert_equal(nil, mail.boundary)

      t.assert_equal("t", Mail.encode_field("t"))
      t.assert_equal("[test:1] Re: =?iso-2022-jp?B?GyRCJUYlOSVIGyhCIA==?= ",
		     Mail.encode_field("[test:1] Re: eXg "))

      # test_ml_submit
      t.assert_equal("test", @name)

      t.assert_equal(" Re: eXg ", Mail.clean_subject("[test:1] Re: eXg "))

      encoded_subject = Mail.rewrite_subject(mail["Subject"], @name, 1)

      t.assert_equal("[test:1] Re: =?iso-2022-jp?B?GyRCJUYlOSVIGyhCIA==?= ",
		     encoded_subject)
#      t.assert_equal(" Re: eXg ", Mail.clean_subject(encoded_subject))
      t.assert_equal("eXg", mail.get_clean_subject)
      t.assert_equal("̓eXgłB\n".sjistojis, mail.get_body)
      t.assert_equal("Mon, 3 Feb 2001 12:34:56 +0900", mail["Date"])
      qwik_post(mail, true)

      t.assert_equal("eXg", @site["1"].get_title)
      t.assert_equal("* eXg\n{{mail(user@e.com,0)\n̓eXgłB\n}}\n", @site["1"].load)

      # 2nd mail
      mail = Mail.new
      mail.mail_from = "guest@example.com"
      mail.add_recipient("test@example.com")
      str = <<"EOT".sjistojis
From: "Guest User" <guest@example.com>
To: "Test Mailing List" <test@example.com>
Subject: Re: [test:2] eXg 
Date: Mon, 4 Feb 2001 12:34:56 +0900

eXgłB
EOT
      mail.read(str)

#      pp @qwikconfig.db
      qwik_post(mail, true)
      t.assert_equal("eXg", @site["1"].get_title)
      t.assert_equal("* eXg\n{{mail(user@e.com,0)\n̓eXgłB\n}}\n{{mail(guest@example.com,0)\neXgłB\n}}\n", @site["1"].load)

      # 3rd mail
      mail = Mail.new
      mail.mail_from = "eto@example.com"
      mail.add_recipient("test@example.com")
      str = <<"EOT".sjistojis
Date: Thu, 15 Apr 2004 00:00:02 +0900
From: Kouichirou Eto <eto@example.com>
To: test@example.com
Subject: i
MIME-Version: 1.0
Content-Type: text/plain; charset="ISO-2022-JP"
Content-Transfer-Encoding: 7bit

eXgł܂B
EOT
      mail.read(str)

      qwik_post(mail, true)
      t.assert_equal("i", @site["2"].get_title)
      t.assert_equal("* i\n{{mail(eto@example.com,0)\neXgł܂B\n}}\n", @site["2"].load)

      # 4th mail
      mail = Mail.new
      mail.mail_from = "eto@example.com"
      mail.add_recipient("test@example.com")
      str = <<"EOM".sjistojis
Date: Thu, 15 Apr 2004 21:00:11 +0900
From: Kouichirou Eto <eto@example.com>
To: test@example.com
Subject: R


EOM
      mail.read(str)

      qwik_post(mail, true)
      t.assert_equal("R", @site["3"].get_title)
      t.assert_equal("* R\n{{mail(eto@example.com,0)\n\n}}\n", @site["3"].load)

      @site.erase_all
      @qwikconfig.db = @org_config_db
    end

    def test_submit_with_attach(t)
      mail = Mail.new
      str = <<"EOT".sjistojis
Date: Sun, 01 Aug 2004 12:34:56 +0900
From: Test User <user@e.com>
To: test@example.com
Subject: multipart test
Cc: guest@example.com
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="boundary"
Content-Transfer-Encoding: 7bit

--boundary
Content-Type: text/plain; charset="ascii"
Content-Transfer-Encoding: 7bit

test1

--boundary
Content-Type: text/plain; charset="ascii"
Content-Transfer-Encoding: 7bit

test2

--boundary--
EOT
      mail.read(str)
      t.assert(mail.multipart?)
      mail0 = Mail.new
      mail0.read mail.parts[0]
      t.assert_equal("test1\n\n", mail0.body)
      mail1 = Mail.new
      mail1.read mail.parts[1]
      t.assert_equal("test2\n\n", mail1.body)
      qwik_post(mail, true)
      t.assert_equal("multipart test", @site["4"].get_title)
      t.assert_equal("* multipart test\n{{mail(,0)\ntest1\ntest2\n}}\n", @site["4"].load)

      mail = Mail.new
      mail.mail_from = "user@e.com"
      mail.add_recipient("test@example.com")
      str = <<"EOT".sjistojis
Date: Sun, 01 Aug 2004 12:34:56 +0900
From: Test User <user@e.com>
To: test@example.com
Subject: =?iso-2022-jp?B?GyRCRTpJVSVGJTklSBsoQg==?=
Cc: guest@example.com
Message-Id: <nosuchmessageid@example.com>
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="------_410DDC04C7AD046D3600_MULTIPART_MIXED_"
Content-Transfer-Encoding: 7bit

--------_410DDC04C7AD046D3600_MULTIPART_MIXED_
Content-Type: text/plain; charset="ISO-2022-JP"
Content-Transfer-Encoding: 7bit

eXgłB

--------_410DDC04C7AD046D3600_MULTIPART_MIXED_
Content-Type: image/png; name="1x1.png"
Content-Disposition: attachment;
 filename="1x1.png"
Content-Transfer-Encoding: base64

iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAADElEQVR42mP4//8/AAX+Av4zEpUU
AAAAAElFTkSuQmCC

--------_410DDC04C7AD046D3600_MULTIPART_MIXED_--
EOT
      mail.read(str)

      # test_mail
      t.assert_equal(["test@example.com"], mail.recipients)
      t.assert_equal(nil, mail.charset)
      t.assert_equal("multipart/mixed", mail.content_type)
      t.assert_equal("user@e.com", mail.mail_from)

      t.assert_equal(str, mail.bare)
      t.assert_equal("YteXg".sjistojis, mail["Subject"])

      t.assert_equal("------_410DDC04C7AD046D3600_MULTIPART_MIXED_", mail.boundary)
      t.assert_equal(2, mail.parts.length)
      mail0 = Mail.new
      mail0.read(mail.parts[0])
      t.assert_equal("text/plain", mail0.content_type)
      t.assert_equal("eXgłB\n\n".sjistojis, mail0.body)

      mail1 = Mail.new
      mail1.read(mail.parts[1])
      t.assert_equal("image/png", mail1.content_type)
      t.assert_equal("attachment;\n filename=\"1x1.png\"", mail1["Content-Disposition"])
      t.assert_equal("base64", mail1["Content-Transfer-Encoding"])
      t.assert_equal("iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAADElEQVR42mP4//8/AAX+Av4zEpUU\nAAAAAElFTkSuQmCC\n\n", mail1.body)
      t.assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000\001\000\000\000\001\010\002\000\000\000\220wS\336\000\000\000\fIDATx\332c\370\377\377?\000\005\376\002\3763\022\225\024\000\000\000\000IEND\256B`\202", mail1.decoded_body)
      t.assert_equal("1x1.png".sjistojis, mail1.filename)

      t.assert_equal(nil, mail.parts[2])

      qwik_post(mail, true)
      t.assert_equal("YteXg", @site["5"].get_title)
      t.assert_equal("* YteXg\n{{mail(user@e.com,0)\neXgłB\n\n{{ref(1x1.png)}}\n\n}}\n", @site["5"].load)
      t.assert_equal(true, @site.attach.exist?("1x1.png"))

      # test attach text file
      mail = Mail.new
      str = <<"EOT".sjistojis
Date: Sun, 01 Aug 2004 12:34:56 +0900
From: 2004@eto.com
To: eto@example.com
Message-Id: <nosuchmessageid@example.com>
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="------_4119D7487CCF04869008_MULTIPART_MIXED_"
Content-Transfer-Encoding: 7bit
Subject: text file attach test
Reply-To: eto@example.com

--------_4119D7487CCF04869008_MULTIPART_MIXED_
Content-Type: text/plain; charset="ISO-2022-JP"
Content-Transfer-Encoding: 7bit

test
--------_4119D7487CCF04869008_MULTIPART_MIXED_
Content-Type: application/octet-stream; name="t.txt"
Content-Disposition: attachment;
 filename="t.txt"
Content-Transfer-Encoding: base64

dGVzdA0K

--------_4119D7487CCF04869008_MULTIPART_MIXED_
EOT
      mail.read(str)
      qwik_post(mail, true)
      page = @site.get_by_title("text file attach test")
      t.assert_equal("text file attach test", page.get_title)
      t.assert_equal("* text file attach test\n{{mail(,0)\ntest\n\n{{ref(t.txt)}}\n\n}}\n", page.load)
      t.assert_equal(true, @site.attach.exist?("t.txt"))
      t.assert_match(%r!test/.attach/t.txt!, @site.attach.path("t.txt").to_s)
      @site.attach.path("t.txt").open {|f| str = f.read }
      t.assert_equal("test\r\n", str)
    end

    def test_uuencode(t)
      mail = Mail.new
      mail.mail_from = "user@e.com"
      mail.add_recipient("test@example.com")
      str = <<'EOT'.sjistojis
Date: Sun, 01 Aug 2004 12:34:56 +0900
From: Test User <user@e.com>
To: test@example.com
Subject: UuencodeImage
Message-Id: <nosuchmessageid@example.com>
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="------------Boundary_iGrnGAlmY.rxNIm"
X-Mailer: EdMax Ver2.85.5F

--------------Boundary_iGrnGAlmY.rxNIm
Content-Type: text/plain; charset=ISO-2022-JP
Content-Transfer-Encoding: 7bit

eXgłB

--------------Boundary_iGrnGAlmY.rxNIm
Content-Type: application/octet-stream; name="1x1a.png"
Content-Transfer-Encoding: x-uuencode
Content-Disposition: attachment; filename="1x1a.png"

begin 666 1x1a.png
MB5!.1PT*&@H````-24A$4@````$````!"`(```"0=U/>````#$E$051XVF/X
8__\_``7^`OXS$I44`````$E%3D2N0F""
`
end
EOT
      mail.read(str)

      # test_mail
      t.assert_equal(["test@example.com"], mail.recipients)
      t.assert_equal(nil, mail.charset)
      t.assert_equal("multipart/mixed", mail.content_type)
      t.assert_equal("user@e.com", mail.mail_from)

      t.assert_equal(str, mail.bare)
      t.assert_equal("UuencodeImage".sjistojis, mail["Subject"])

      t.assert_equal("------------Boundary_iGrnGAlmY.rxNIm", mail.boundary)
      t.assert_equal(2, mail.parts.length)
      mail0 = Mail.new
      mail0.read(mail.parts[0])
      t.assert_equal("text/plain", mail0.content_type)
      t.assert_equal("eXgłB\n\n".sjistojis, mail0.body)

      mail1 = Mail.new
      mail1.read(mail.parts[1])
      t.assert_equal("application/octet-stream", mail1.content_type)
      t.assert_equal("attachment; filename=\"1x1a.png\"", mail1["Content-Disposition"])
      t.assert_equal("x-uuencode", mail1["Content-Transfer-Encoding"])
      uu = "MB5!.1PT*&@H````-24A\$4@````\$````!\"`(```\"0=U/>````#\$E\$051XVF/X\n8__\\_``7^`OXS\$I44`````\$E%3D2N0F\"\"\n"
      t.assert_equal("begin 666 1x1a.png\n#{uu}`\nend\n", mail1.body)
      t.assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000\001\000\000\000\001\010\002\000\000\000\220wS\336\000\000\000\fIDATx\332c\370\377\377?\000\005\376\002\3763\022\225\024\000\000\000\000IEND\256B`\202", mail1.decoded_body)
      t.assert_equal("1x1a.png".sjistojis, mail1.filename)
      t.assert_equal(nil, mail.parts[2])

      qwik_post(mail, true)
      page = @site["UuencodeImage"]
      t.assert_equal("UuencodeImage", page.get_title)
      t.assert_equal("* UuencodeImage\n{{mail(user@e.com,0)\neXgłB\n\n{{ref(1x1a.png)}}\n\n}}\n", page.load)
    end

    def test_search(t)
      mail = Mail.new
      mail.mail_from = "user@e.com"
      mail.add_recipient("test@example.com")
      str = <<'EOT'.sjistojis
Date: Sun, 01 Aug 2004 12:34:56 +0900
From: Test User <user@e.com>
To: test@example.com
Subject: UuencodeImage
Message-Id: <nosuchmessageid@example.com>
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="------------Boundary_iGrnGAlmY.rxNIm"
X-Mailer: EdMax Ver2.85.5F

--------------Boundary_iGrnGAlmY.rxNIm
Content-Type: text/plain; charset=ISO-2022-JP
Content-Transfer-Encoding: 7bit

eXgłB

--------------Boundary_iGrnGAlmY.rxNIm
Content-Type: application/octet-stream; name="1x1a.png"
Content-Transfer-Encoding: x-uuencode
Content-Disposition: attachment; filename="1x1a.png"

begin 666 1x1a.png
MB5!.1PT*&@H````-24A$4@````$````!"`(```"0=U/>````#$E$051XVF/X
8__\_``7^`OXS$I44`````$E%3D2N0F""
`
end
EOT
      mail.read(str)

      # test_mail
      t.assert_equal(["test@example.com"], mail.recipients)
      t.assert_equal(nil, mail.charset)
      t.assert_equal("multipart/mixed", mail.content_type)
      t.assert_equal("user@e.com", mail.mail_from)

      t.assert_equal(str, mail.bare)
      t.assert_equal("UuencodeImage".sjistojis, mail["Subject"])

      t.assert_equal("------------Boundary_iGrnGAlmY.rxNIm", mail.boundary)
      t.assert_equal(2, mail.parts.length)
      mail0 = Mail.new
      mail0.read(mail.parts[0])
      t.assert_equal("text/plain", mail0.content_type)
      t.assert_equal("eXgłB\n\n".sjistojis, mail0.body)

      mail1 = Mail.new
      mail1.read(mail.parts[1])
      t.assert_equal("application/octet-stream", mail1.content_type)
      t.assert_equal("attachment; filename=\"1x1a.png\"", mail1["Content-Disposition"])
      t.assert_equal("x-uuencode", mail1["Content-Transfer-Encoding"])
      uu = "MB5!.1PT*&@H````-24A\$4@````\$````!\"`(```\"0=U/>````#\$E\$051XVF/X\n8__\\_``7^`OXS\$I44`````\$E%3D2N0F\"\"\n"
      t.assert_equal("begin 666 1x1a.png\n#{uu}`\nend\n", mail1.body)
      t.assert_equal("\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000\001\000\000\000\001\010\002\000\000\000\220wS\336\000\000\000\fIDATx\332c\370\377\377?\000\005\376\002\3763\022\225\024\000\000\000\000IEND\256B`\202", mail1.decoded_body)
      t.assert_equal("1x1a.png".sjistojis, mail1.filename)
      t.assert_equal(nil, mail.parts[2])

      qwik_post(mail, true)
      page = @site["UuencodeImage"]
      t.assert_equal("UuencodeImage", page.get_title)
#     t.assert_equal("* UuencodeImage\n{{mail(user@e.com,0)\neXgłB\n\n{{ref(1x1a.png)}}\n\n}}\n", page.load)
    end

    undef_method :close if method_defined?(:close)
    def close
      @logger.log "[#{@name}]: ML Closed"
    end
  end

  class Logger
#   undef_method :puts_log if method_defined?(:puts_log)
    undef puts_log
    def puts_log(msg) # for test
      return if msg =~ /New ML/
      time = Time.now.strftime("%Y-%m-%dT%H:%M:%S")
      open("testlog.txt", "ab"){|f|
	f.puts "#{time}: #{msg}"
      }
    end
  end

  class Sweeper
    def test(t)
#      t.assert_equal(true, ml_file?("/qwik/data/test"))
#      sweep
    end
  end
end

class TC_QuickML < Test::Unit::TestCase
  def setup
    if !defined?($quickml_config) || $quickml_config.nil?
      confstr = File.safe_open("../../bin/quickmlrc").read
      confhash = eval(confstr)
      confhash[:data_dir] = "."
      $quickml_config  = QuickML::Config.new(confhash)
    end
    @config = $quickml_config
    @dir = @config.data_dir.path+"test"
    @dir.teardown
    @catalog = @config.catalog
    @message_charset = "iso-2022-jp"
  end

  def teardown
    @dir.teardown
  end

  def test_quickml
    qml = QuickML::QuickML.new(@config, "test@example.com")

    # test private
    qml.test(self)

    # test check mail address
    assert(QuickML::QuickML.valid_name?("test@example.com"))
    assert(!QuickML::QuickML.valid_name?("test@qwik@jp"))
    assert(QuickML::QuickML.valid_name?("test"))
    assert(!QuickML::QuickML.valid_name?("te.st"))

    # test_sweeper
    sweeper = QuickML::Sweeper.new(@config)
    assert_instance_of(QuickML::Sweeper, sweeper)
    sweeper.test(self)

    # test_encode_field
    assert_equal("t", QuickML::Mail.encode_field("t"))
    assert_equal("=?iso-2022-jp?B?GyRCJCIbKEI=?= ",
		 QuickML::Mail.encode_field(""))
    assert_equal("=?iso-2022-jp?B?GyRCJCIkJBsoQg==?= ",
		 QuickML::Mail.encode_field(""))
    assert_equal("=?iso-2022-jp?B?GyRCJCIkJCQmJCgkKiQrJC0kLyQxJDMbKEI=?= \n =?iso-2022-jp?B?GyRCJDUkNyQ5JDskPRsoQg==?= ", QuickML::Mail.encode_field(""))
    assert_equal("=?iso-2022-jp?B?GyRCJCIbKEI=?= ",
		 QuickML::Mail.encode_field(""))
    assert_equal("\201@", "@")
    assert_equal("\352\242", "")
    assert_equal("[test:1] Re: =?iso-2022-jp?B?GyRCJUYlOSVIGyhCICA=?= ",
		 QuickML::Mail.encode_field("[test:1] Re: eXg  "))
#   assert_equal("", QuickML::Mail.encode_field("[test:1] Re: \245\306\245\271\245\310 "))

    # test submit
    qml.test_submit(self)
    qml.test_submit_with_attach(self)
    qml.test_uuencode(self)
    qml.test_search(self)
  end
end

class TC_QuickML_GetText < Test::Unit::TestCase
  include QuickML::GetText::GetText

  def assert_jis(e, s)
    assert_equal(e.sjistojis, s)
  end

  def test_dummy
  end

  def nutest_gettext_ja
    assert_jis("g: a\n", _("Info: %s\n", "a"))
    assert_jis("g: a\n", gettext("Info: %s\n", "a"))
    assert_jis("FꍇA\n", _("If you agree, then,\n"))
  end

  def nutest_gettext_en
    assert_jis("Info: a\n", _("Info: %s\n", "a"))
    assert_jis("Info: a\n", gettext("Info: %s\n", "a"))
    assert_jis("If you agree, then,\n", _("If you agree, then,\n"))
  end
end

class TC_Uuencode < Test::Unit::TestCase
  def test_uuencode
    assert_equal("", [""].pack("u"))
    assert_equal("!80``\n", ["a"].pack("u"))
    assert_equal("#86)C\n", ["abc"].pack("u"))
    assert_equal("$86)C9```\n", ["abcd"].pack("u"))
    assert_equal("M86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A\n", ["a"*45].pack("u"))
    assert_equal("M86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A\n!80``\n", ["a"*46].pack("u"))
    assert_equal("&86)C9&5F\n#9VAI\n", ["abcdefghi"].pack("u6"))

    assert_equal([""], "".unpack("u"))
    assert_equal([""], "".unpack("u"))
    assert_equal(["a"], "!80``\n".unpack("u"))
    assert_equal(["abc"], "#86)C\n".unpack("u"))
    assert_equal(["abcd"], "$86)C9```\n".unpack("u"))
    assert_equal(["a"*45], "M86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A\n".unpack("u"))
    assert_equal(["a"*46], "M86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A86%A\n!80``\n".unpack("u"))
    assert_equal(["abcdefghi"], "&86)C9&5F\n#9VAI\n".unpack("u6"))

    png = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000\001\000\000\000\001\010\002\000\000\000\220wS\336\000\000\000\fIDATx\332c\370\377\377?\000\005\376\002\3763\022\225\024\000\000\000\000IEND\256B`\202"
    uu = "MB5!.1PT*&@H````-24A\$4@````\$````!\"`(```\"0=U/>````#\$E\$051XVF/X\n8__\\_``7^`OXS\$I44`````\$E%3D2N0F\"\"\n"
    assert_equal(uu, [png].pack("u"))
    assert_equal([png], uu.unpack("u"))
  end
end
